@@ -1,6 +1,4 @@
-/*
-#ident "@(#)smail/pd/sdbm:RELEASE-3_2_0_121:pair.c,v 1.4 2003/03/27 16:49:22 woods Exp"
- */
+/* @(#) pair.c,v 1.3 1992/08/01 20:02:20 tron Exp */
 /*
  * sdbm - ndbm work-alike hashed database library
  * based on Per-Aake Larson's Dynamic Hashing algorithms. BIT 18 (1978).
@@ -11,23 +9,26 @@
  */
 
 #ifndef lint
-static char rcsid[] = "pair.c,v 1.4 2003/03/27 16:49:22 woods Exp";
+static char rcsid[] = "pair.c,v 1.3 1992/08/01 20:02:20 tron Exp";
 #endif
 
-#include "defs.h"				/* Smail-3 */
-
-#include <sys/types.h>
-
-#include "tune.h"
+#include "defs.h"
 #include "sdbm.h"
+#include "tune.h"
 #include "pair.h"
 
+#ifdef __STDC__
+#define P_(p) p
+#else
+#define P_(p) ()
+#endif
+
 #define exhash(item)	dbm_hash((item).dptr, (item).dsize)
 
 /* 
  * forward 
  */
-static int seepair __P((char *, int, char *, int));
+static int seepair P_((char *, int, char *, int));
 
 /*
  * page format:
@@ -83,13 +84,13 @@ datum val;
  * enter the key first
  */
 	off -= key.dsize;
-	(void) memcpy(pag + off, key.dptr, (size_t) key.dsize);
+	(void) memcpy(pag + off, key.dptr, key.dsize);
 	ino[n + 1] = off;
 /*
  * now the data
  */
 	off -= val.dsize;
-	(void) memcpy(pag + off, val.dptr, (size_t) val.dsize);
+	(void) memcpy(pag + off, val.dptr, val.dsize);
 	ino[n + 2] = off;
 /*
  * adjust item count
@@ -199,7 +200,7 @@ datum key;
 		}
 #else
 #if (defined (HAVE_MEMMOVE) || defined(ANSI_C)) && !defined(NO_HAVE_MEMMOVE)
-		memmove(dst, src, (size_t) m);
+		memmove(dst, src, m);
 #else
 		while (m--)
 			*--dst = *--src;
@@ -235,7 +236,7 @@ register int siz;
 
 	for (i = 1; i < n; i += 2) {
 		if (siz == off - ino[i] &&
-		    memcmp(key, pag + ino[i], (size_t) siz) == 0)
+		    memcmp(key, pag + ino[i], siz) == 0)
 			return i;
 		off = ino[i + 1];
 	}
