@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_121:direct.c,v 1.56 2005/08/26 19:59:40 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2:direct.c,v 1.14 1996/02/28 14:26:19 woods Exp"
  */
 
 /*
@@ -20,64 +20,26 @@
  *			    find_direct_driver, read_director_file.
  *                          dump_director_config
  */
-
-#include "defs.h"
-
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <stdio.h>
-#include <errno.h>
 #include <pwd.h>
 #include <grp.h>
-
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif
-
-#ifdef HAVE_STRING_H
-# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
-#  include <memory.h>
-# endif
-# include <string.h>
-#endif
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif
-
-#ifdef __STDC__
-# include <stdarg.h>
-#else
-# include <varargs.h>
-#endif
-
-#if defined(HAVE_UNISTD_H)
-# include <unistd.h>
-#endif
-
+#include <sys/types.h>
+#include <sys/stat.h>
+#include "defs.h"
 #include "smail.h"
-#include "config.h"
-#include "main.h"
-#include "parse.h"
-#include "addr.h"
 #include "direct.h"
-#include "directors/user.h"
+#include "transport.h"
 #include "log.h"
-#include "alloc.h"
-#include "list.h"
-#include "smailstring.h"
+#include "addr.h"
+#include "main.h"
 #include "dys.h"
 #include "exitcodes.h"
-#include "transport.h"
 #include "smailconf.h"
-#include "debug.h"
-#include "extern.h"
-#include "error.h"
-#include "smailport.h"
+#ifndef DEPEND
+# include "debug.h"
+# include "extern.h"
+# include "error.h"
+#endif
 
 /* enums local to this file */
 enum local_form {
@@ -96,31 +58,33 @@ enum other_form_op {
 int cached_directors = FALSE;		/* TRUE if cache_directors() called */
 
 /* functions local to this file */
-static void implicit_redirect __P((struct addr*, struct addr **, char *));
-static enum other_form_op finish_direct_other_form __P((struct director *, struct addr *));
-static void finish_direct_special_form __P((struct addr *, struct director *, enum local_form,
-					    struct addr **, struct addr **, struct addr **,
-					    struct addr **, struct addr **));
-static void cache_director_data __P((struct director *));
-static void premunge_local_addrs __P((struct addr *));
-static enum local_form addr_type __P((char *));
-static char *director_driv_function __P((char *, struct attribute *));
+static void implicit_redirect();
+static enum other_form_op finish_direct_other_form();
+static void finish_direct_special_form();
+static void cache_director_data();
+static void premunge_local_addrs();
+static enum local_form addr_type();
+static int verify_special();
+static char *director_driv_function();
 
 /* Configuration template information */
 static struct attr_table director_generic[] = {
-    { "driver", t_string, 0, NULL, NULL, OFFSET(director, driver) },
-    { "caution", t_boolean, 0, NULL, NULL, CAUTION_DIRECTOR },
-    { "nobody", t_boolean, 0, NULL, NULL, NOBODY_DIRECTOR },
-    { "sender_okay", t_boolean, 0, NULL, NULL, SENDER_OKAY },
-    { "ignore_alias_match", t_boolean, 0, NULL, NULL, IGNORE_ALIAS_MATCH },
-    { "owner", t_string, 0, NULL, NULL, OFFSET(director, owner) },
-    { "default_user", t_string, 0, NULL, NULL, OFFSET(director, default_user) },
-    { "default_group", t_string, 0, NULL, NULL, OFFSET(director, default_group) },
-    { "default_home", t_string, 0, NULL, NULL, OFFSET(director, default_home) },
-    { "domains", t_string, 0, NULL, NULL, OFFSET(director, domains) },
-    { "set_user", t_string, 0, NULL, NULL, OFFSET(director, set_user) },
-    { "set_group", t_string, 0, NULL, NULL, OFFSET(director, set_group) },
-    { "set_home", t_string, 0, NULL, NULL, OFFSET(director, set_home) },
+    { "driver", t_string, NULL, NULL, OFFSET(director, driver) },
+    { "caution", t_boolean, NULL, NULL, CAUTION_DIRECTOR },
+    { "nobody", t_boolean, NULL, NULL, NOBODY_DIRECTOR },
+    { "sender_okay", t_boolean, NULL, NULL, SENDER_OKAY },
+    { "ignore_alias_match", t_boolean, NULL, NULL, IGNORE_ALIAS_MATCH },
+    { "owner", t_string, NULL, NULL, OFFSET(director, owner) },
+    { "default_user", t_string, NULL, NULL,
+	  OFFSET(director, default_user) },
+    { "default_group", t_string, NULL, NULL,
+	  OFFSET(director, default_group) },
+    { "default_home", t_string, NULL, NULL,
+	  OFFSET(director, default_home) },
+    { "domains", t_string, NULL, NULL, OFFSET(director, domains) },
+    { "set_user", t_string, NULL, NULL, OFFSET(director, set_user) },
+    { "set_group", t_string, NULL, NULL, OFFSET(director, set_group) },
+    { "set_home", t_string, NULL, NULL, OFFSET(director, set_home) },
 };
 
 
@@ -192,7 +156,7 @@ direct_local_addrs(in, out, retry, defer, fail)
 				 defer,
 				 note_error(ERR_CONFERR|ERR_107,
 					    xprintf(
-				     "director %v: driver %s driver not found",
+				     "director %s: driver %s driver not found",
 						    dp->name,
 						    dp->driver)));
 		exitvalue = EX_DATAERR;
@@ -201,47 +165,40 @@ direct_local_addrs(in, out, retry, defer, fail)
 	    /* initialize list of new addrs found in this pass */
 	    newaddrs = NULL;
 
-	    /* If a domain list exists, remove from the input list all those
-	     * addresses that are not for the domains handled by this
-	     * director.  Afterwards, put them back on the input list.
-	     */
+            /* If a domain list exists, remove from the input list all those
+            addresses that are not for the domains handled by this
+            director. Afterwards, put them back on the input list. */
+
             other_domains = NULL;
-	    if (dp->domains) {
-		struct addr *temp = in;
-		struct addr **prev = &in;
+            if (dp->domains) {
+                struct addr *temp = in;
+                struct addr **prev = &in;
 
-		while (temp != NULL) {
-		    if (!temp->local_name || !is_string_in_list(temp->local_name, dp->domains)) {
-			*prev = temp->succ;
-			temp->succ = other_domains;
-			other_domains = temp;
-			temp = *prev;
-		    } else {
-			prev = &(temp->succ);
-			temp = temp->succ;
-		    }
-		}
-	    }
-
-	    /* call the driver if any input addresses are left to direct */
-	    if (in != NULL) {
-		in = (*driver->driver)(dp, in, out, &newaddrs, defer, fail);
-	    }
-
-	    /* put back any that were held back because they did not match the
-	     * director's domain list
-	     */
-	    if (other_domains != NULL) {
-		if (in == NULL) {
-		    in = other_domains;
-		} else {
-		    insert_addr_list(other_domains, &in, (struct error *) NULL);
-		}
+                while (temp != NULL) {
+                    if (!temp->local_name ||
+                      !is_string_in_list(temp->local_name, dp->domains)) {
+                        *prev = temp->succ;
+                        temp->succ = other_domains;
+                        other_domains = temp;
+                        temp = *prev;
+                    } else {
+                        prev = &(temp->succ);
+                        temp = temp->succ;
+                    }
+                }
             }
 
-	    /* step through newaddrs to verify them (newaddrs will be left as
-	     * NULL if the input address(es) had ADDR_VRFY_ONLY set)
-	     */
+            /* call the driver if any left */
+            if (in != NULL)
+              in = (*driver->driver)(dp, in, out, &newaddrs, defer, fail);
+
+            /* put back any that were held back */
+            if (other_domains != NULL) {
+                if (in == NULL) in = other_domains; else
+                   insert_addr_list(other_domains, &in, (struct error *) NULL);
+            }
+
+	    /* step through newaddrs to verify them */
 	    for (cur = newaddrs; cur; cur = next) {
 		enum local_form form = addr_type(cur->work_addr);
 
@@ -310,20 +267,19 @@ direct_local_addrs(in, out, retry, defer, fail)
 	for (; in; in = next) {
 	    next = in->succ;
 
-	    /* ... unless they're super-special... */
-	    if (EQIC(in->remainder, "mailer-daemon")) {
-		implicit_redirect(in, retry, postmaster_address);
-	    } else if (EQIC(in->remainder, "postmaster")) {
+	    if (EQIC(in->remainder, "MAILER-DAEMON")) {
+		implicit_redirect(in, retry, "Postmaster");
+	    } else if (EQIC(in->remainder, "Postmaster")) {
 		implicit_redirect(in, retry, postmaster_address);
 	    } else {
-		/* no director found the mailbox, fail the address */
-		exitvalue = EX_NOUSER;	/* also set exit status, just in case */
+		/* no director found the user, fail the address */
+		exitvalue = EX_NOUSER;	/* also set exit status */
 		/*
 		 * ERR_100 - unknown user
 		 *
 		 * DESCRIPTION
-		 *	The remainder string in this addr structure was not
-		 *	matched by any of the directors.
+		 *	The remainder structure in this addr structure was
+		 *      not matched by any of the directors.
 		 *
 		 * ACTIONS
 		 *      Send a message to the owner of the address, or to
@@ -333,9 +289,9 @@ direct_local_addrs(in, out, retry, defer, fail)
 		 *      A list owner should check through the list that he
 		 *      or she owns.  A sender may wish to send mail to the
 		 *      postmaster of the machine asking about login names
-		 *      or aliases for a particular user.
+		 *      for a particular user.
 		 */
-		in->error = note_error(ERR_NSOWNER|ERR_100, "no such user or mailbox");
+		in->error = note_error(ERR_NSOWNER|ERR_100, "unknown user");
 		in->succ = *fail;
 		*fail = in;
 	    }
@@ -349,6 +305,132 @@ direct_local_addrs(in, out, retry, defer, fail)
 	includes = NULL;
     }
 }
+
+
+/*
+ * verify_local_addrs - perform quick verify of local addresses
+ *
+ * form a list of okay (verified) addrs, plus deferred (not currently
+ * determinable) addrs and failed (not deliverable) addrs.
+ */
+void
+verify_local_addrs(in, okay, defer, fail)
+    struct addr *in;			/* input local addr list */
+    struct addr **okay;			/* output list of verified addrs */
+    struct addr **defer;		/* temporariliy unverifiable addrs */
+    struct addr **fail;			/* unverified addrs */
+{
+    struct director *dp;		/* temp for stepping thru directors */
+    register struct addr *cur;		/* temp for stepping through addrs */
+    struct addr *next;			/* next value for cur */
+    struct addr *other_domains;		/* for domain-specific handling */
+
+    DEBUG(DBG_ROUTE_HI, "verify_local_addrs called\n");
+
+    /* do any munging of the addr structures required before directing */
+    premunge_local_addrs(in);
+
+    /*
+     * give the complete input list to each director in turn.
+     */
+    for (dp = directors; in && dp; dp = dp->succ) {
+	/* look up the director driver by name */
+	struct direct_driver *driver = find_direct_driver(dp->driver);
+	struct addr *retry;		/* set of addrs for next director */
+
+	if (driver == NULL) {
+	    /*
+	     * ERR_107 - director driver not found
+	     *
+	     * DESCRIPTION
+	     *      A driver name was not found in the table of director
+	     *      drivers.
+	     *
+	     * ACTIONS
+	     *      Defer all remaining input addresses with configuration
+	     *      errors.  Since it is not known if this director would
+	     *      have matched any addresses, we cannot pass the addresses
+	     *      on to the next director.  Thus, we must defer all
+	     *      addresses not matched by a previous director.
+	     *
+	     * RESOLUTION
+	     *      The postmaster must check the director and transport
+	     *      configurations before delivery can be performed.
+	     */
+	    insert_addr_list(in,
+			     defer,
+			     note_error(ERR_CONFERR|ERR_107,
+					xprintf(
+						"director %s: driver %s driver not found",
+						dp->name,
+						dp->driver)));
+	    return;
+	}
+
+        /* If a domain list exists, remove from the input list all those
+	   addresses that are not for the domains handled by this
+	   director. Afterwards, put them back on the input list. */
+
+        other_domains = NULL;
+        if (dp->domains) {
+            struct addr *temp = in;
+            struct addr **prev = &in;
+
+            while (temp != NULL) {
+                if (!temp->local_name ||
+		    !is_string_in_list(temp->local_name, dp->domains)) {
+                    *prev = temp->succ;
+                    temp->succ = other_domains;
+                    other_domains = temp;
+                    temp = *prev;
+                } else {
+                    prev = &(temp->succ);
+                    temp = temp->succ;
+                }
+            }
+        }
+
+        /* call the driver if any left */
+        retry = NULL;
+        if (in != NULL)
+	    (*driver->verify)(dp, in, &retry, okay, defer, fail);
+
+        /* put back any that were held back */
+        if (other_domains != NULL) {
+            if (retry == NULL) retry = other_domains; else
+		insert_addr_list(other_domains, &retry, (struct error *) NULL);
+        }
+
+        in = retry;
+    }
+
+    for (cur = in; cur; cur = next) {
+	next = cur->succ;
+	/* no director found the user, fail the address */
+	exitvalue = EX_NOUSER;		/* also set exit status */
+	/*
+	 * ERR_100 - unknown user
+	 *
+	 * DESCRIPTION
+	 *      The remainder structure in this addr structure was not
+	 *      matched by any of the directors.
+	 *
+	 * ACTIONS
+	 *      Send a message to the owner of the address, or to the
+	 *      sender if there is no owner.
+	 *
+	 * RESOLUTION
+	 *      A list owner should check through the list that he or
+	 *      she owns.  A sender may wish to send mail to the
+	 *      postmaster of the machine asking about login names for a
+	 *      particular user.
+	 */
+	cur->error = note_error(ERR_NSOWNER|ERR_100, "unknown user");
+	cur->succ = *fail;
+	*fail = cur;
+    }
+}
+
 
 /*
  * cache_directors - call cache entrypoints for all directors
@@ -421,7 +503,7 @@ implicit_redirect(match, retry, in_addr)
     new->succ = *retry;
     *retry = new;
     DEBUG3(DBG_DIRECT_LO,
-	   "implicity matched special mailbox <%v>\n    redirected <%v> --> <%v>\n",
+	   "smail implicity matched %s\n    directed %s --> %s\n",
 	   match->in_addr, match->in_addr, in_addr);
 }
 
@@ -447,19 +529,12 @@ finish_direct_other_form(dp, addr)
 {
     int parseflags;
     int escaped = 0;
-    char *temp_target = NULL;
-    char *temp_remainder;
-    char *temp_work_addr;
-    int form;
 
-    if (!addr->parent) {
-	panic(EX_SOFTWARE, "finish_direct_other_form(): director %v: no parent address for <%v>.", dp->name, addr->in_addr);
-	/* NOTREACHED */
-    }
     /* not an address form we need to be worried about */
     if (EQIC(addr->work_addr, addr->parent->remainder) ||
 	(addr->work_addr[0] == '\\' &&
-	 EQIC(addr->work_addr + 1, addr->parent->remainder))) {
+	 EQIC(addr->work_addr + 1, addr->parent->remainder)))
+    {
 	/* for the form \parent-addr drop the \ */
 	if (addr->work_addr[0] == '\\') {
 	    escaped = 1;
@@ -474,129 +549,94 @@ finish_direct_other_form(dp, addr)
 	 * director, just drop the address.
 	 */
 	if (! escaped && (dp->flags & IGNORE_ALIAS_MATCH)) {
-	    DEBUG3(DBG_DIRECT_LO,
-		  "    directed <%v> --> <%v> using %v ... (matched parent addr, not escaped, but ignore_alias_match set), passing to next director\n",
+	    DEBUG2(DBG_DIRECT_LO,
+		  "    directed %s --> %s ... matched, pass to next director\n",
 		   addr->parent->remainder,
-		   addr->work_addr,
-		   dp->name);
+		   addr->work_addr);
 	    return DROP_ADDRESS;
 	}
-	DEBUG4(DBG_DIRECT_LO,
-	       "    directed <%v> --> %s<%v> using %v ... matched parent addr, pass to next director\n",
+	DEBUG2(DBG_DIRECT_LO,
+	       "    directed %s --> %s ... matched, pass to next director\n",
 	       addr->parent->remainder,
-	       escaped ? "\\" : "",
-	       addr->work_addr,
-	       dp->name);
+	       addr->work_addr);
 	addr->remainder = addr->work_addr;
 	return SEND_TO_NEXT;
-    }
-    /*
-     * XXX - reparse the address looking for references
-     *	 to the the localhost with a remainder equal
-     *	 to the parent address.  This allows the use
-     *	 of username@localhost addresses in alias
-     *	 files, making it possible to share an alias
-     *	 file for redirecting users around a network.
-     *
-     *	 This moves intelligence out of resolve.c that
-     *	 really belongs there.
-     */
-
-    temp_work_addr = COPY_STRING(addr->work_addr);
-    parseflags = addr->parseflags;
-    form = parse_address(temp_work_addr, &temp_target, &temp_remainder,
-			 &parseflags);
-
-    switch (form) {
-    case FAIL:
-	xfree(temp_work_addr);
-	break;
-
-    case LOCAL:
+    } else {
 	/*
-	 * if the me_too flag is not set, then filter the sender
-	 * from the output of directors.  A director can
-	 * explicitly allow sender addresses to be produced by
-	 * including the `sender_okay' attribute.  When expanding
-	 * an address list for verification purposes, the sender
-	 * may not be specified.  Watch out for this case.
+	 * XXX - reparse the address looking for references
+	 *	 to the the localhost with a remainder equal
+	 *	 to the parent address.  This allows the use
+	 *	 of username@localhost addresses in alias
+	 *	 files, making it possible to share an alias
+	 *	 file for redirecting users around a network.
+	 *
+	 *	 This moves intelligence out of resolve.c that
+	 *	 really belongs there.
 	 */
-	xfree(temp_work_addr);
-	if (! (dp->flags & SENDER_OKAY) &&
-	    ! me_too &&
-	    sender &&
-	    EQIC(sender, addr->work_addr))
-	{
-	    DEBUG3(DBG_DIRECT_LO,
-		   "    directed <%v> --> <%v> using %v ... matched sender, ignored\n",
-		   addr->parent->remainder, addr->work_addr, dp->name);
-	    return DROP_ADDRESS;
-	}
-	break;
+	char *temp_target;
+	char *temp_remainder;
+	char *temp_work_addr;
+	int form;
 
-    default:
-	if (islocalhost(temp_target) &&
-	    EQIC(temp_remainder, addr->parent->remainder) &&
-	    ((addr->parent->local_name && EQIC(temp_target, addr->parent->local_name)) ||
-	     (addr->parent->target && EQIC(temp_target, addr->parent->target))))
-	{
-	    /*
-	     * new addr same as input addr, send to
-	     * the next director.  E.g., foo aliased
-	     * to foo.
-	     */
-	    DEBUG5(DBG_DIRECT_LO,
-		   "    directed <%v@%v> --> <%v> using %v ... matched localhost, passing %v to next director\n",
-		   addr->parent->remainder,
-		   addr->parent->target ? addr->parent->target : "(no-parent-target)",
-		   addr->work_addr,
-		   dp->name,
-		   temp_remainder);
-	    (void) strcpy(addr->work_addr, temp_remainder); /* XXXXXX does this clobber???? */
-	    addr->remainder = addr->work_addr;
+	temp_work_addr = COPY_STRING(addr->work_addr);
+	parseflags = addr->parseflags;
+	form = parse_address(temp_work_addr, &temp_target, &temp_remainder,
+			     &parseflags);
+
+	switch (form) {
+	case FAIL:
+	case PARSE_ERROR:
 	    xfree(temp_work_addr);
+	    break;
 
-	    return SEND_TO_NEXT;
+	case LOCAL:
+	    /*
+	     * if the me_too flag is not set, then filter the sender
+	     * from the output of directors.  A director can
+	     * explicitly allow sender addresses to be produced by
+	     * including the `sender_okay' attribute.  When expanding
+	     * an address list for verification purposes, the sender
+	     * may not be specified.  Watch out for this case.
+	     */
+	    xfree(temp_work_addr);
+	    if (! (dp->flags & SENDER_OKAY) &&
+		! me_too &&
+		sender &&
+		EQIC(sender, addr->work_addr))
+	    {
+		DEBUG2(DBG_DIRECT_LO,
+		       "    directed %s --> %s ... match sender, ignore\n",
+		       addr->parent->remainder,
+		       addr->work_addr);
+		return DROP_ADDRESS;
+	    }
+	    break;
+
+	default:
+	    if (islocalhost(temp_target) &&
+		EQIC(temp_remainder, addr->parent->remainder))
+	    {
+		/*
+		 * new addr same as input addr, send to
+		 * the next director.  E.g., foo aliased
+		 * to foo.
+		 */
+		DEBUG2(DBG_DIRECT_LO,
+		   "    directed %s --> %s ... match, pass to next director\n",
+		       addr->parent->remainder,
+		       addr->work_addr);
+		(void) strcpy(addr->work_addr, temp_remainder);
+		addr->remainder = addr->work_addr;
+		xfree(temp_work_addr);
+		return SEND_TO_NEXT;
+	    }
 	}
-	/* Turning on ADDR_DONTHASH prevents accidental matching of remainder
-	 * for virtual hosts or for cases where an alias/etc. re-directs to a
-	 * localhost.
-	 */
-	addr->flags |= ADDR_DONTHASH;
-	DEBUG7(DBG_DIRECT_HI,
-	       "    NOT passing %v@%v (%v) to next director as <%v>%s%s%s\n",
-	       addr->parent->remainder,
-	       addr->parent->target ? addr->parent->target : "(no-parent-target)",
-	       addr->parent->local_name ? addr->parent->local_name : "(no-parent-local_name)",
-	       addr->work_addr,
-	       islocalhost(temp_target) ? "" :  ", not a localhost",
-	       EQIC(temp_remainder, addr->parent->remainder) ? "" : ", not same remainder",
-	       (addr->parent->local_name && EQIC(temp_target, addr->parent->local_name)) ||
-	       (addr->parent->target && EQIC(temp_target, addr->parent->target)) ? "" : ", not same target");
     }
 
     /* we have a new address to reparse */
-#ifndef NDEBUG
-    if (debug) {
-	char *in_addr = COPY_STRING(addr->in_addr);
-
-	/* this is a cheap hack, but it's better than nothing! */
-	if ((strncmpic(in_addr, ":defer:", sizeof(":defer:")-1) == 0) ||
-	    (strncmpic(in_addr, ":error:", sizeof(":error:")-1) == 0) ||
-	    (strncmpic(in_addr, ":fail:", sizeof(":fail:")-1) == 0)) {
-
-	    *(strchr(in_addr + 1, ':') + 1) = '\0';
-	}
-
-	DEBUG4(DBG_DIRECT_LO, "    directed <%v@%v> --> <%v> using %v\n",
-	       addr->parent->remainder,
-	       addr->parent->target ? addr->parent->target : "(localhost)",
-	       in_addr,
-	       dp->name);
-
-	xfree(in_addr);
-    }
-#endif
+    DEBUG2(DBG_DIRECT_LO, "    directed %s --> %s\n",
+	   addr->parent->remainder,
+	   addr->in_addr);
 
     return ADDRESS_OKAY;
 }
@@ -610,7 +650,7 @@ finish_direct_special_form(addr, dp, form, new, out, defer, fail, includes)
     struct addr *addr;			/* addr structure to finish */
     struct director *dp;		/* matching director */
     enum local_form form;		/* local address form */
-    struct addr **new __attribute__((unused));	/* put new addrs to retry here */
+    struct addr **new;			/* put new addrs to retry here */
     struct addr **out;			/* resolved addrs */
     struct addr **defer;		/* put config errors here */
     struct addr **fail;			/* put failed addrs here */
@@ -652,7 +692,7 @@ finish_direct_special_form(addr, dp, form, new, out, defer, fail, includes)
 	 *      named in the director.
 	 */
 	addr->error = note_error(ERR_NPOWNER|ERR_104,
-				 xprintf("director %v: security violation",
+				 xprintf("director %s: security violation",
 					 dp->name));
 	addr->succ = *fail;
 	*fail = addr;
@@ -663,8 +703,8 @@ finish_direct_special_form(addr, dp, form, new, out, defer, fail, includes)
 	if (operation_mode != VERIFY_ADDRS &&
 	    operation_mode != TEST_MODE)
 	{
-	    write_log(WRITE_LOG_SYS,
-		"%v ... director %v: child of %v insecure, access as 'nobody'",
+	    write_log(LOG_SYS,
+		"%s ... director %s: child of %s insecure, access as 'nobody'",
 		      addr->in_addr,
 		      dp->name,
 		      addr->parent->in_addr);
@@ -681,10 +721,8 @@ finish_direct_special_form(addr, dp, form, new, out, defer, fail, includes)
 	 * pipe form, associate with the "pipe" transport
 	 */
 	if (pipe_transport == NULL) {
-	    pipe_transport = find_transport(PIPE_TRANSPORT_NM);
+	    pipe_transport = find_transport("pipe");
 	    if (pipe_transport == NULL) {
-		char *errtxt;
-
 		/* configuration error, try again later */
 		/*
 		 * ERR_105 - no pipe transport
@@ -701,23 +739,21 @@ finish_direct_special_form(addr, dp, form, new, out, defer, fail, includes)
 		 *      The postmaster should add a "pipe"
 		 *      transport to the transport file.
 		 */
-		errtxt = xprintf("no `pipe' transport with the name `%s'", PIPE_TRANSPORT_NM);
-		addr->error = note_error(ERR_CONFERR | ERR_105, errtxt);
+		addr->error = note_error(ERR_CONFERR|ERR_105,
+					 "no pipe transport");
 		addr->succ = *defer;
 		*defer = addr;
 		break;
 	    }
 	}
-	addr->transport = pipe_transport;
 
+	addr->transport = pipe_transport;
 	/* setup command as $user variable for transports */
 	addr->next_addr = COPY_STRING(addr->work_addr + 1);
-	DEBUG4(DBG_DIRECT_LO,
-	       "    directed <%v> --> <%v> using %v ... send to the %s transport\n",
+	DEBUG2(DBG_DIRECT_LO,
+	       "    directed %s --> %s ... send to pipe transport\n",
 	       addr->parent->remainder,
-	       addr->in_addr,
-	       dp->name,
-	       PIPE_TRANSPORT_NM);
+	       addr->in_addr);
 	/* addr finished link into the output queue */
 	addr->succ = *out;
 	*out = addr;
@@ -728,10 +764,8 @@ finish_direct_special_form(addr, dp, form, new, out, defer, fail, includes)
 	 * file form, associate with the "file" transport
 	 */
 	if (file_transport == NULL) {
-	    file_transport = find_transport(FILE_TRANSPORT_NM);
+	    file_transport = find_transport("file");
 	    if (file_transport == NULL) {
-		char *errtxt;
-
 		/* configuration error, must be fixed */
 		/*
 		 * ERR_106 - no file transport
@@ -748,8 +782,8 @@ finish_direct_special_form(addr, dp, form, new, out, defer, fail, includes)
 		 *      The postmaster should add a "file"
 		 *      transport to the transport file.
 		 */
-		errtxt = xprintf("no `appendfile' transport with the name `%s'", FILE_TRANSPORT_NM);
-		addr->error = note_error(ERR_CONFERR | ERR_106, errtxt);
+		addr->error = note_error(ERR_CONFERR|ERR_106,
+					 "no file transport");
 		addr->succ = *defer;
 		*defer = addr;
 		break;
@@ -759,12 +793,10 @@ finish_direct_special_form(addr, dp, form, new, out, defer, fail, includes)
 
 	/* setup file as $user variable for transports */
 	addr->next_addr = COPY_STRING(addr->work_addr);
-	DEBUG4(DBG_DIRECT_LO,
-	       "    directed <%v> --> <%v> using %v ... send to %s transport\n",
+	DEBUG2(DBG_DIRECT_LO,
+	       "    directed %s --> %s ... send to file transport\n",
 	       addr->parent->remainder,
-	       addr->in_addr,
-	       dp->name,
-	       FILE_TRANSPORT_NM);
+	       addr->in_addr);
 	/* addr finished link into the output queue */
 	addr->succ = *out;
 	*out = addr;
@@ -772,23 +804,15 @@ finish_direct_special_form(addr, dp, form, new, out, defer, fail, includes)
 
     case INCLUDE_FORM:
 	/* mailing lists go back through all the directors */
-	DEBUG3(DBG_DIRECT_LO,
-	       "    directed <%v> --> <%v> using %v ... mailing list\n",
+	DEBUG2(DBG_DIRECT_LO,
+	       "    directed %s --> %s ... mailing list\n",
 	       addr->parent->remainder,
-	       addr->in_addr,
-	       dp->name);
+	       addr->in_addr);
 	/* set local part to the mailing list address */
 	addr->remainder = addr->work_addr;
 	addr->succ = *includes;
 	*includes = addr;
 	break;
-
-    case OTHER_FORM:
-	panic(EX_SOFTWARE,
-	      "director %v, impossible situation:  reached OTHER_FORM",
-	      dp->name);
-	/* NOTREACHED */
-	break;
     }
 }
 
@@ -805,8 +829,7 @@ cache_director_data(dp)
     register struct director *dp;
 {
     if (dp->default_user) {
-	struct passwd *pw = getpwbyname(FALSE, dp->default_user);
-
+	struct passwd *pw = getpwbyname(dp->default_user);
 	if (pw) {
 	    dp->default_uid = pw->pw_uid;
 	    dp->default_gid = pw->pw_gid;
@@ -816,7 +839,6 @@ cache_director_data(dp)
     }
     if (dp->default_group) {
 	struct group *gr = getgrbyname(dp->default_group);
-
 	if (gr) {
 	    dp->default_gid = gr->gr_gid;
 	} else {
@@ -829,7 +851,6 @@ cache_director_data(dp)
 			  (struct addr *)NULL,
 			  (char *)NULL,
 			  (char *)NULL);
-
 	if (x_home && !EQ(x_home, dp->default_home)) {
 	    dp->x_default_home = COPY_STRING(x_home);
 	} else {
@@ -837,8 +858,7 @@ cache_director_data(dp)
 	}
     }
     if (dp->set_user) {
-	struct passwd *pw = getpwbyname(FALSE, dp->set_user);
-
+	struct passwd *pw = getpwbyname(dp->set_user);
 	if (pw) {
 	    dp->set_uid = pw->pw_uid;
 	    dp->set_gid = pw->pw_gid;
@@ -848,7 +868,6 @@ cache_director_data(dp)
     }
     if (dp->set_group) {
 	struct group *gr = getgrbyname(dp->set_group);
-
 	if (gr) {
 	    dp->set_gid = gr->gr_gid;
 	} else {
@@ -861,7 +880,6 @@ cache_director_data(dp)
 			  (struct addr *)NULL,
 			  (char *)NULL,
 			  (char *)NULL);
-
 	if (x_home && !EQ(x_home, dp->set_home)) {
 	    dp->x_set_home = COPY_STRING(x_home);
 	} else {
@@ -986,8 +1004,6 @@ addr_type(s)
 /*
  * director_user_info - fill an addr structure with user information
  *
- * XXX This function should probably be in directors/user.c, though it is also
- * used in directors/fwdfile.c
  *
  * if the remainder field if the addr structure matches a username,
  * fill in fields which pertain to the user's passwd file entry.
@@ -997,23 +1013,25 @@ director_user_info(addr)
     struct addr *addr;			/* addr structure to check */
 {
     struct passwd *pw;			/* passwd file entry for user */
+    char *dupe = NULL;			/* ptr to lowercased remainter */
 
-    if (addr->flags & (ADDR_NOTUSER | ADDR_ISUSER)) {
+    if (addr->flags & (ADDR_NOTUSER|ADDR_ISUSER)) {
 	/* a previous call to director_user_info() already took care of this */
 	return;
     }
 
     /* get the passwd entry if one exists */
-    pw = getpwbyname((int) ignore_user_case, addr->remainder);
+    pw = getpwbyname(addr->flags & USER_IGNORE_CASE ?
+		     str2lower((dupe = COPY_STRING(addr->remainder))) :
+		     addr->remainder);
+    if (dupe)
+	    free(dupe);
     if (pw) {
 	/* passwd entry found */
 	addr->flags |= ADDR_ISUSER;
 	addr->uid = pw->pw_uid;
 	addr->gid = pw->pw_gid;
 	addr->home = COPY_STRING(pw->pw_dir);
-	if (ignore_user_case) {
-	    strcpy(addr->remainder, pw->pw_name); /* preserve case */
-	}
 	return;
     }
 
@@ -1078,7 +1096,6 @@ read_director_file()
     FILE *f;				/* open director file */
     char *error;			/* error from read_standard_file() */
     struct stat statbuf;
-    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     struct attr_table *end_director_generic = ENDTABLE(director_generic);
     static struct director director_template = {
 	NULL,				/* name */
@@ -1102,6 +1119,8 @@ read_director_file()
 	NULL,				/* cache - x_set_home */
     };
 
+    extern int errno;
+
     /*
      * try to open directory file, stat file if possible
      */
@@ -1111,7 +1130,7 @@ read_director_file()
     f = fopen(director_file, "r");
     if (f == NULL) {
 	if (require_configs) {
-	    return xprintf("%v: %s", director_file, strerror(errno));
+	    return xprintf("%s:%s", director_file, strerror(errno));
 	}
 
 	add_config_stat(director_file, (struct stat *)NULL);
@@ -1122,8 +1141,8 @@ read_director_file()
     add_config_stat(director_file, &statbuf);
 
     /* call read_standard_file to do the real work */
-    error = read_standard_file(f, director_file,
-			       (char *) &director_template,
+    error = read_standard_file(f,
+			       (char *)&director_template,
 			       sizeof(struct director),
 			       OFFSET(director, name),
 			       OFFSET(director, flags),
@@ -1131,14 +1150,14 @@ read_director_file()
 			       director_generic,
 			       end_director_generic,
 			       director_driv_function,
-			       (char **) &directors);
+			       (char **)&directors);
 
     /* finish up */
     (void) fclose(f);
 
     /* return any error message */
     if (error) {
-	return xprintf("%v: %s", director_file, error);
+	return xprintf("%s: %s", director_file, error);
     }
     return NULL;
 }
@@ -1151,12 +1170,13 @@ director_driv_function(struct_p, driver_attrs)
     struct director *dp = (struct director *)struct_p;
     struct direct_driver *drv;
 
+    DEBUG(DBG_ROUTE_HI, "director_driv_function called\n");
     if (dp->driver == NULL) {
-	return xprintf("director %v: no driver attribute", dp->name);
+	return xprintf("director %s: no driver attribute", dp->name);
     }
     drv = find_direct_driver(dp->driver);
     if (drv == NULL) {
-	return xprintf("director %v: unknown driver: %s",
+	return xprintf("director %s: unknown driver: %s",
 		       dp->name, dp->driver);
     }
     if (drv->builder) {
@@ -1171,10 +1191,9 @@ dump_director_config(f)
      FILE * f;
 {
     struct director *dp;		/* temp for stepping thru directors */
-    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     struct attr_table *end_director_generic = ENDTABLE(director_generic);
     char * name;			/* name of director */
-    unsigned long flags;		/* flags for director */
+    long flags;				/* flags for director */
     struct direct_driver *driver;	/* drivers for director */
 
     DEBUG(DBG_ROUTE_HI, "dump director configs called\n");
@@ -1186,7 +1205,7 @@ dump_director_config(f)
     for (dp = directors; dp; dp = dp->succ) {
 	name = dp->name;
 	flags = dp->flags;
-	fprintf(f, "#\n%s:\n", name);	/* not %v since reading back can't decode */
+	fprintf(f, "#\n%s:\n", name);
 	(void) dump_standard_config(f,
 				    (char *) dp,
 				    name,
@@ -1195,21 +1214,8 @@ dump_director_config(f)
 				    end_director_generic);
 	fputs("\t;\n", f);
 	driver = find_direct_driver(dp->driver);
-	if (driver) {
-	    if (driver->dumper) {
-		(*driver->dumper)(f, dp);
-	    } else {
-		fprintf(f, "\t#\n\t# -- the %s director driver has no 'dumper' function!\n\t#\n", dp->driver);
-	    }
-	} else {
-	    fprintf(f, "\t#\n\t# -- there is no %s director driver!\n\t#\n", dp->driver);
-	}
+	if (driver->dumper)
+	    (*driver->dumper)(f, dp);
     }
     fputs("#\n# -- end of directors\n", f);
 }
-
-/* 
- * Local Variables:
- * c-file-style: "smail"
- * End:
- */
