@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_120:hash.c,v 1.28 2004/02/02 03:20:18 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2:hash.c,v 1.7 1996/02/28 06:47:07 woods Exp"
  */
 
 /*
@@ -30,73 +30,50 @@
  *		write_hash_table, walk_hash, free_hash_element,
  *		free_hash_table
  */
-
-#include "defs.h"
-
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
-
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif
-#ifdef HAVE_STRING_H
-# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
-#  include <memory.h>
-# endif
-# include <string.h>
-#endif
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif
-#ifdef __STDC__
-# include <stdarg.h>
-#else
-# include <varargs.h>
-#endif
-
 #include "smail.h"
-#include "alloc.h"
-#include "list.h"
-#include "smailarch.h"
-#include "main.h"
-#include "addr.h"
+#include "defs.h"
 #include "exitcodes.h"
 #include "hash.h"
-#include "smailstring.h"
-#include "dys.h"
-#include "log.h"
-#include "extern.h"
-#include "debug.h"
-#include "smailport.h"
+#include "alloc.h"
+#ifndef DEPEND
+# include "extern.h"
+# include "debug.h"
+#endif
 
 /* static functions used in this file */
-static int hash_str __P((char *, unsigned int));
-static int hash_stric __P((char *, unsigned int));
-static struct hash *new_hash_element __P((char *, char *, size_t, struct hash_table *));
+static int hash_str();
+static int hash_stric();
+static struct hash *new_hash_element();
 
 #ifdef STANDALONE
 int debug = 0;	/* default debug level is NONE */
-# define errfile stderr
-extern void panic();
-# define xmalloc(x) (malloc(x))		/* naive allocator */
-# define bmalloc(x,y) (malloc(x))	/* naive allocator */
-# define xfree(x) (free(x))		/* naive deallocator */
-# define bfree(x,y) (free(x))		/* naive deallocator */
-/* functions imported by this file */
-# if !defined(HAVE_STDLIB_H) && !defined(HAVE_MALLOC_H)
-extern char *malloc();
-extern char *realloc();
-extern void free();				/* int on SunOS */
-# endif
+#define errfile stderr
+void panic();
+#define xmalloc(x) (malloc(x))	/* naive allocator */
+#define bmalloc(x,y) (malloc(x))	/* naive allocator */
+char *malloc();
+#define xfree(x) (free(x))	/* naive deallocator */
+#define bfree(x,y) (free(x))	/* naive deallocator */
+void free();
 #endif	/* STANDALONE */
 
+/*
+ * ETHEREAL_HASHDATA -
+ *
+ *    This file contains a nearly complete implementation of a general
+ *    hashing system.  However it is likely that smail itself will
+ *    never need to delete, replace, or store new data over old data.
+ *    That is, smail will only create, add, lookup and perform hash
+ *    table file operations.
+ *
+ *    The code ifdef-ed out does work and is tested by the STANDALONE code.
+ *    You should define ETHEREAL_HASHDATA if you want to use it
+ */
+
+
 /*
  * hash_str - convert a trying into a hash value
  *
@@ -126,7 +103,7 @@ extern void free();				/* int on SunOS */
 static int
 hash_str(str, mod)
     register char *str;			/* the string to hash */
-    unsigned int mod;			/* prime modulus, size of hash table */
+    int mod;				/* prime modulus, size of hash table */
 {
     register unsigned long val;		/* current hash value */
     register unsigned long excess;	/* upper/excess bits in val */
@@ -145,21 +122,21 @@ hash_str(str, mod)
      */
     if (mod < HASH_LEVEL) {
 	/* hash each character using the L_ values */
-	for (val = 0; (c = (unsigned long) *str); ++str) {
+	for (val = 0; c=(unsigned long)*str; ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val & L_HASH_MASK)) >> L_DOWN_SHIFT);
+	    val ^= ((excess = (val&L_HASH_MASK)) >> L_DOWN_SHIFT);
 	    val ^= excess;
 	}
     } else {
 	/* hash each character using the H_ values */
-	for (val = 0; (c = (unsigned long) *str); ++str) {
+	for (val = 0; c=(unsigned long)*str; ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val & H_HASH_MASK)) >> H_DOWN_SHIFT);
+	    val ^= ((excess = (val&H_HASH_MASK)) >> H_DOWN_SHIFT);
 	    val ^= excess;
 	}
     }
     
-    return (int) (val % mod);    /* our hash value, mod the hash size */
+    return (int)(val%mod);    /* our hash value, mod the hash size */
 }
 
 /*
@@ -191,7 +168,7 @@ hash_str(str, mod)
 static int
 hash_stric(str, mod)
     register char *str;		/* the string to hash disregarding case */
-    unsigned int mod;		/* prime modulus, size of hash table */
+    int mod;			/* prime modulus, size of hash table */
 {
     register unsigned long val;		/* current hash value */
     register unsigned long excess;	/* upper/excess bits in val */
@@ -210,24 +187,25 @@ hash_stric(str, mod)
      */
     if (mod < HASH_LEVEL) {
 	/* hash each character using the L_ values */
-	for (val = 0; (c = (unsigned long) tolower((int) *str)); ++str) {
+	for (val = 0; c=(unsigned long)lowercase(*str); ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val & L_HASH_MASK)) >> L_DOWN_SHIFT);
+	    val ^= ((excess = (val&L_HASH_MASK)) >> L_DOWN_SHIFT);
 	    val ^= excess;
 	}
     } else {
 	/* hash each character using the H_ values */
-	for (val = 0; (c = (unsigned long) tolower((int) *str)); ++str) {
+	for (val = 0; c=(unsigned long)lowercase(*str); ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val & H_HASH_MASK)) >> H_DOWN_SHIFT);
+	    val ^= ((excess = (val&H_HASH_MASK)) >> H_DOWN_SHIFT);
 	    val ^= excess;
 	}
     }
     
-    return (int) (val % mod);    /* our hash value, mod the hash size */
+    return (int)(val%mod);    /* our hash value, mod the hash size */
 }
 
 
+#ifdef ETHEREAL_HASHDATA
 /*
  * walk_hash - walk thru a hash table
  *
@@ -251,8 +229,8 @@ walk_hash(cur, table)
     struct hash_table *table;	/* hash table being walked */
 {
     register struct hash *prev;	/* our previous walk location */
-    register unsigned int indx;	/* our previous hash slot */
-    register unsigned int len;	/* the table length */
+    register int indx;		/* our previous hash slot */
+    register int len;		/* the table length */
 
     /*
      * firewall
@@ -334,6 +312,7 @@ walk_hash(cur, table)
     table->indx = indx;
     return table->prev;
 }
+#endif	/* ETHEREAL_HASHDATA */
 
 
 /*
@@ -351,11 +330,11 @@ static struct hash *
 new_hash_element(keystr, data, datalen, table)
     char *keystr;    			/* the keystring to be added */
     char *data;				/* the associated data if any */
-    size_t datalen;			/* length of data,  0 ==> no data */
+    int datalen;			/* length of data,  0 ==> no data */
     struct hash_table *table;		/* hash table being added to */
 {
     struct hash *new;		/* the new slot chain location */
-    size_t keylen;		/* the length of the string, padded */
+    int keylen;			/* the length of the string, padded */
     unsigned long lk;		/* temp var for key length */
 
     /*
@@ -365,22 +344,22 @@ new_hash_element(keystr, data, datalen, table)
 	panic(EX_SOFTWARE, "new_hash_element: NULL keystring or table");
     }
     lk = keystr_len(keystr);		/* compute padded key length */
-    if (lk >= (unsigned long) (1L << BITS_PER_SHORT)) {
+    if (lk >= (unsigned long)(1L<<BITS_PER_SHORT)) {
 	panic(EX_SOFTWARE, "new_hash_element: key too long");
     }
-    keylen = (size_t) lk;		/* now we know it will fit in an int */
+    keylen = (int)lk;		/* now we know it will fit in an int */
     /* firewall - check against bad data being passed to us */
-    if ((datalen > 0 && data == NULL) || 
-	datalen >= (unsigned long) (1L << BITS_PER_SHORT))  {
+    if (datalen < 0 || (datalen > 0 && data == NULL) || 
+      (unsigned long)datalen >= (unsigned long)(1L<<BITS_PER_SHORT))  {
 	panic(EX_SOFTWARE,
-	    "new_hash_element: bad data passed with: <%s>  datalen: <%lu>",
+	    "new_hash_element: bad data passed with: <%s>  datalen: <%d>",
 	    keystr, datalen);
     }
 
     /*
      * malloc the storage
      */
-    new = (struct hash *) bmalloc((size_t) hashslot_size(keylen, datalen), table->life);
+    new = (struct hash *)bmalloc( hashslot_size(keylen,datalen), table->life );
     /* firewall */
     if (is_odd(new)) {
 	panic(EX_SOFTWARE,
@@ -401,6 +380,7 @@ new_hash_element(keystr, data, datalen, table)
     return new;
 }
 
+#ifdef ETHEREAL_HASHDATA
 /*
  * free_hash_element - free an old hash element
  *
@@ -427,7 +407,7 @@ new_hash_element(keystr, data, datalen, table)
 void
 free_hash_element(cur, search, table)
     struct hash *cur;		/* what we will delete */
-    unsigned int search;	/* non-zero => delete `cur' first */
+    int search;			/* non-zero => delete `cur' first */
     struct hash_table *table;	/* table `cur' does/would_have belonged to */
 {
     /*
@@ -447,12 +427,10 @@ free_hash_element(cur, search, table)
     /*
      * free the storage
      */
-    if (!table->life) {
-	DEBUG(DBG_HASH_LO, "free_hash_element(): about to panic() with NULL block\n");
-    }
-    bfree((char *) cur, table->life);
+    bfree(cur, table->life);
     return;
 }
+#endif	/* ETHEREAL_HASHDATA */
 
 /*
  * new_hash_table - creat a new hash table
@@ -468,27 +446,29 @@ free_hash_element(cur, search, table)
  */
 struct hash_table *
 new_hash_table(tablelen, life, flags)
-    unsigned int tablelen;	/* number of slots in the hash table */
+    int tablelen;		/* number of slots in the hash table */
     struct block *life;		/* is the hash table permanent or temporary */
     int flags;			/* hash table flag as per struct hash_table */
 {
-    register unsigned int i;		/* index */
+    register int i;			/* index */
     struct hash_table *table;		/* the malloced hash table */
 
     /*
      * firewalls
      */
+    if (tablelen <= 0) {
+	panic(EX_SOFTWARE, "new_hash_table: tablelen: %d", tablelen);
+    }
     DEBUG3(DBG_HASH_LO,
-	   "new_hash_table: tablelen: %u life: 0x%lx flags: 0x%x\n", tablelen, (long)life, flags);
+	   "new_hash_table: tablelen:%d life:%d flag:%d\n",tablelen,life,flags);
 
     /*
      * malloc the hash table
      */
-    table = (struct hash_table *) bmalloc(table_size(tablelen), life);
-
+    table = (struct hash_table *)bmalloc(table_size(tablelen), life);
     /* firewall */
     if (is_odd(table)) {
-	panic(EX_SOFTWARE, "new_hash_table: malloc returned odd address: 0x%lx", (long)table);
+	panic(EX_SOFTWARE, "new_hash_table: malloc returned odd address: %ld", (long)table);
     }
 
     /*
@@ -505,6 +485,7 @@ new_hash_table(tablelen, life, flags)
     return table;	/* return our new table */
 }
 
+#ifdef ETHEREAL_HASHDATA
 /*
  * free_hash_table - free a hash table and its associated data
  *
@@ -519,6 +500,8 @@ void
 free_hash_table(table)
     struct hash_table *table;	/* the hash table to free */
 {
+    struct hash *cur;		/* current element to delete */
+
     /*
      * firewalls
      */
@@ -530,15 +513,10 @@ free_hash_table(table)
     /*
      * free the table slots
      */
-    if (!table->life) {
-	DEBUG(DBG_HASH_LO, "free_hash_table(): about to panic() with NULL block\n");
-    }
-    bfree((char *) table, table->life);
-
-    /* XXX why don't we free 'table' itself?  or free_block() it? */
-
+    bfree(table, table->life);
     return;
 }
+#endif	/* ETHEREAL_HASHDATA */
 
 
 /*
@@ -558,7 +536,7 @@ int
 add_to_hash(keystr, data, datalen, table)
     char *keystr;    			/* the keystring to be added */
     char *data;				/* the associated data if any */
-    size_t datalen;			/* length of data,  0 ==> no data */
+    int datalen;			/* length of data,  0 ==> no data */
     struct hash_table *table;		/* the hash table to add it to */
 {
     register struct hash *cur;		/* the current slot chain location */
@@ -599,7 +577,7 @@ add_to_hash(keystr, data, datalen, table)
     /* 
      * search the chain
      */
-    DEBUG2(DBG_HASH_VHI, "add_to_hash: slot: 0x%lx cur: 0x%lx\n",
+    DEBUG2(DBG_HASH_VHI, "add_to_hash: slot:0x%lx cur:0x%lx\n",
 	   (long)table->slot[loc], (long)table->slot[loc]);
     for (prev = NULL, cur = table->slot[loc];
 	 cur != NULL;
@@ -607,7 +585,7 @@ add_to_hash(keystr, data, datalen, table)
 	/* 
 	 * if we found the entry, stop
 	 */
-	DEBUG2(DBG_HASH_VHI, "add_to_hash: comparing <%s> to <%s>\n",
+	DEBUG2(DBG_HASH_VHI, "add_to_hash: comparing <%s> to <%s>",
 	       keystr, cur->keystr);
 	if ((cmp = stringcmp(keystr, cur->keystr, caseflag)) == 0) {
 	    DEBUG(DBG_HASH_MID, "add_to_hash: already hashed\n");
@@ -639,6 +617,7 @@ add_to_hash(keystr, data, datalen, table)
     return JUST_HASHED;
 }
 
+#ifdef ETHEREAL_HASHDATA
 /*
  * replace_in_hash - replace an existing element in a hash table
  *
@@ -655,7 +634,7 @@ struct hash *
 replace_in_hash(keystr, data, datalen, table)
     char *keystr;    			/* the keystring to replace */
     char *data;				/* the associated data if any */
-    size_t datalen;			/* length of data,  0 ==> no data */
+    int datalen;			/* length of data,  0 ==> no data */
     struct hash_table *table;		/* the hash table to add it to */
 {
     register struct hash *cur;		/* the current slot chain location */
@@ -739,7 +718,7 @@ struct hash *
 store_in_hash(keystr, data, datalen, table)
     char *keystr;    			/* the keystring to replace */
     char *data;				/* the associated data if any */
-    size_t datalen;			/* length of data,  0 ==> no data */
+    int datalen;			/* length of data,  0 ==> no data */
     struct hash_table *table;		/* the hash table to add it to */
 {
     register struct hash *cur;		/* the current slot chain location */
@@ -822,6 +801,7 @@ store_in_hash(keystr, data, datalen, table)
     insert_hash(&prev->succ, new);
     return NULL;
 }
+#endif	/* ETHEREAL_HASHDATA */
 
 /*
  * lookup_in_hash - lookup an element in a hash table and return 
@@ -844,12 +824,12 @@ int
 lookup_in_hash(keystr, data, datalen, table)
     char *keystr;			/* the key to lookup */
     char **data;			/* where to point at data, or NULL */
-    size_t *datalen;			/* where to place data len or NULL */
+    int *datalen;			/* where to place data len or NULL */
     struct hash_table *table;		/* the hash table to add it to */
 {
     register struct hash *cur;	/* the slot chain location */
-    unsigned int loc;		/* the hash slot to add onto */
-    unsigned int caseflag;	/* 0 ==> use strcmp, 1 ==> strcmpic */
+    int loc;			/* the hash slot to add onto */
+    int caseflag;		/* 0 ==> use strcmp, 1 ==> strcmpic */
     int cmp;			/* compare function result */
 
     /*
@@ -904,6 +884,7 @@ lookup_in_hash(keystr, data, datalen, table)
     return NOT_HASHED;
 }
 
+#ifdef ETHEREAL_HASHDATA
 /*
  * delete_from_hash - delete an element in the hash table
  *
@@ -921,9 +902,9 @@ delete_from_hash(keystr, table)
 {
     register struct hash *cur;		/* the slot chain location */
     register struct hash *prev;		/* previous element */
-    unsigned int loc;			/* the hash slot to add onto */
-    unsigned int caseflag;		/* 0 ==> use strcmp, 1 ==> strcmpic */
-    int cmp;				/* compare function result */
+    int loc;			/* the hash slot to add onto */
+    int caseflag;		/* 0 ==> use strcmp, 1 ==> strcmpic */
+    int cmp;			/* compare function result */
 
     /*
      * firewall - watch for NULLs
@@ -978,6 +959,7 @@ delete_from_hash(keystr, table)
     DEBUG(DBG_HASH_MID, "delete_from_hash: not found\n");
     return NULL;		/* nothing was deleted */
 }
+#endif	/* ETHEREAL_HASHDATA */
 
 
 /*
@@ -992,11 +974,11 @@ write_hash_table(table, stream)
     struct hash_table *table;	/* the table to write */
     FILE *stream;		/* the stream on which to write table */
 {
-    register size_t i;		/* index */
-    off_t tab_loc;		/* file location of hash_table start */
+    register int i;		/* index */
+    long tab_loc;		/* file location of hash_table start */
     long loc;			/* current location */
     int size;			/* size of the current element */
-    off_t offset;		/* current offset within the file */
+    long offset;                /* current offset within the file */
     long disc_succ;             /* cur->succ as written to disc */
     struct hash *cur;		/* the current hash element */
     struct hash_table *tab;	/* disk copy of table */
@@ -1010,8 +992,8 @@ write_hash_table(table, stream)
     if (table->len <= 0) {
 	panic(EX_SOFTWARE, "write_hash_table: table length: %d", table->len);
     }
-    DEBUG2(DBG_HASH_LO, "write_hash_table: size: %d life: 0x%lx\n",
-	   table->len, (long)table->life);
+    DEBUG2(DBG_HASH_LO, "write_hash_table: size: %d life: %d\n",
+	   table->len, table->life);
 
     /*
      * allocate a temporary disk copy of the hash table
@@ -1027,9 +1009,9 @@ write_hash_table(table, stream)
      * skip the hash table block
      */
     tab_loc = ftell(stream);
-    if (fseek(stream, (long int) table_size(table->len), 1) < 0) {
+    if (fseek(stream, (long)table_size(table->len), 1) < 0) {
 	panic(EX_IOERR, "write_hash_table: bad skip of %d over table",
-	      table_size(table->len));
+	    table_size(table->len));
     }
 
     /*
@@ -1040,14 +1022,14 @@ write_hash_table(table, stream)
 	/* don't write out chains that don't exist */
 	if (empty_slot(table->slot[i])) {
 		/* slot is empty, deal with it quickly */
-		set_ptr(tab->slot[i], (int) NULL);
+		set_ptr(tab->slot[i], (int)NULL);
 		continue;
 	}
 
 	/* note starting offset of hash chain */
 	offset = ftell(stream) - tab_loc;
 	if (is_odd(offset)) {
-		panic(EX_IOERR, "write_hash_table: slot %d offset is odd: 0x%lx",
+		panic(EX_IOERR, "write_hash_table: slot %d offset is odd:%ld",
 		      i, offset);
 	}
 	set_ptr(tab->slot[i], to_odd(offset));
@@ -1058,20 +1040,20 @@ write_hash_table(table, stream)
 	    size = hash_len(cur);
 	    if (is_odd(size)) {
 		    panic(EX_IOERR,
-			  "write_hash_table: size is odd: 0x%lx for <%s>",
+			  "write_hash_table: size is odd:%ld for <%s>",
 			  offset, cur->keystr);
 	    }
 	    offset += size;		/* note file movement */
 	    /* write the hash element disk successor element */
 	    disc_succ = (cur->succ == NULL) ? 0 : to_odd(offset);
-	    if (fwrite((char *)&disc_succ, sizeof(disc_succ), (size_t) 1, stream) != 1) {
+	    if (fwrite((char *)&disc_succ, sizeof(disc_succ), 1, stream) != 1) {
 		panic(EX_IOERR,
 		      "write_hash_table: bad succ write <%s> on slot %d",
 		      cur->keystr, i);
 	    }
 	    /* write the rest of the hash element data */
 	    if (fwrite((char *)cur+sizeof(cur->succ), size-sizeof(cur->succ),
-		       (size_t) 1, stream) != 1) {
+		       1, stream) != 1) {
 		panic(EX_IOERR,
 		      "write_hash_table: bad write <%s> on slot %d",
 		      cur->keystr, i);
@@ -1084,14 +1066,15 @@ write_hash_table(table, stream)
      * current position
      */
     loc = ftell(stream);	/* remember our current location */
-    if (fseek(stream, (long int) tab_loc, 0) < 0) {
-	panic(EX_IOERR, "write_hash_table: bad skip back to table at 0x%lx", (long)tab_loc);
+    if (fseek(stream, (long)tab_loc, 0) < 0) {
+	panic(EX_IOERR, "write_hash_table: bad skip back to table at %d",
+	    tab_loc);
     }
-    if (fwrite((char *)tab, table_size(tab->len), (size_t) 1, stream) != 1) {
+    if (fwrite((char *)tab, table_size(tab->len), 1, stream) != 1) {
 	panic(EX_IOERR, "write_hash_table: bad table write");
     }
-    if (fseek(stream, (long int) loc, 0) < 0) {
-	panic(EX_IOERR, "write_hash_table: bad end seek to 0x%lx", (long)loc);
+    if (fseek(stream, (long)loc, 0) < 0) {
+	panic(EX_IOERR, "write_hash_table: bad end seek to %d", loc);
     }
 
     /*
@@ -1106,36 +1089,11 @@ write_hash_table(table, stream)
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <limits.h>
-#include "dys.h"
 
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
+#define TABLE_LEN 3 /* use only a few slots to stress the chain code */
+#define INPUT_SIZE (70*1024)	/* max input line */
 
-#ifdef USE_VMALLOC
-# include <vmalloc.h>
-# define malloc_check()	vmdbcheck(Vmheap)
-#endif
-
-#if !defined(HAVE_STDLIB_H) && !defined(HAVE_MALLOC_H)
-extern char *mktemp();
-#endif
-
-#ifndef malloc_check
-# define malloc_check()	/* NO-OP */
-#endif
-
-#if !defined(SIZE_T_MAX) && defined(UINT_MAX)
-# define SIZE_T_MAX	UINT_MAX	/* most modern systems use UINT_MAX */
-#elif !defined(SIZE_T_MAX)
-# define SIZE_T_MAX	0xffffffffU	/* XXX 'U' is not a valid suffix in traditional C */
-#endif
-
-#define TABLE_LEN	3		/* use only a few slots to stress the chain code */
-
-int return_to_sender = FALSE;
-char *tempname = "/tmp/smhtXXXXXX";	/* tempory hash file name */
+char *tempname = "/tmp/hashtestXXXXXX";	/* tempory hash file name */
 
 void
 main(argc, argv)
@@ -1143,10 +1101,8 @@ main(argc, argv)
     char *argv[]; 	/* args */
 {
     int i;			/* index */
-    char *buf;			/* the input buffer for stdin args */
+    char buf[INPUT_SIZE+1];	/* the input buffer for stdin args */
     struct hash *cur;		/* pointer to walk the table */
-    struct block *table_block;	/* bmalloc() for bfree() */
-    struct block *tableic_block; /* bmalloc() for bfree() */
     struct hash_table *table;	/* our allocated table */
     struct hash_table *tableic;	/* allocated table without regard to case */
     void test();		/* test an with an element */
@@ -1157,7 +1113,7 @@ main(argc, argv)
      * establish debug level
      */
     DEBUG(DBG_HASH_LO, "main: start\n");
-    if (argc > 1 && strncmp(argv[1], "-v", 2) == 0) {
+    if (argc > 1 && strncmp(argv[1], "-d", 2) == 0) {
 	    /* we have a debug level, use it */
 	    if (argv[1][2] != '\0') {
 		    debug = atoi(&argv[1][2]);
@@ -1174,33 +1130,26 @@ main(argc, argv)
     /*
      * setup a hash table
      */
-    table_block = malloc_block();
-    table = new_hash_table(TABLE_LEN, table_block, HASH_CASEFOLD);
-    tableic_block = malloc_block();
-    tableic = new_hash_table(TABLE_LEN, tableic_block, 0);
+    table = new_hash_table(TABLE_LEN, (struct block *)NULL, HASH_CASEFOLD);
+    tableic = new_hash_table(TABLE_LEN, (struct block *)NULL, 0);
 
     /*
      * special case: no args means read one arg per line
      */
     if (argc == 1) {
-	while ((buf = read_line(stdin))) {
-	    int len;
-
-	    /* trim the trailing newline, if any */
-	    len = strlen(buf);
-	    if (buf[len - 1] == '\n') {
-		buf[len - 1] = '\0';
-	    }
+	while(fgets(buf, INPUT_SIZE, stdin) != NULL) {
+	    i = strlen(buf);
+	    buf[i-1] = '\0';
 	    DEBUG1(DBG_HASH_LO,
 		   "main: testing <%s> -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-\n", buf);
 	    test(buf, table);
-	    if (debug >= DBG_HASH_HI) {
+	    if ( debug >= DBG_HASH_HI ) {
 		dump_hash_table(table);
 	    }
 	    DEBUG1(DBG_HASH_LO,
 		   "main: testing ignore case <%s> -*-*-*-*-*-*-*-*-*-\n", buf);
 	    test(buf, tableic);
-	    if (debug >= DBG_HASH_HI) {
+	    if ( debug >= DBG_HASH_HI ) {
 		dump_hash_table(tableic);
 	    }
 	}
@@ -1209,17 +1158,17 @@ main(argc, argv)
      * hash each argument
      */
     } else {
-	for (i = 1; i < argc; ++i) {
+	for (i=1; i < argc; ++i) {
 	    DEBUG1(DBG_HASH_LO,
 		   "main: testing <%s> -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-\n", buf);
 	    test(argv[i], table);
-	    if (debug >= DBG_HASH_HI) {
+	    if ( debug >= DBG_HASH_HI ) {
 		dump_hash_table(table);
 	    }
 	    DEBUG1(DBG_HASH_LO,
 		   "main: testing ignore case <%s> -*-*-*-*-*-*-*-*-*-\n", buf);
 	    test(argv[i], tableic);
-	    if (debug >= DBG_HASH_HI) {
+	    if ( debug >= DBG_HASH_HI ) {
 		dump_hash_table(tableic);
 	    }
 	}
@@ -1238,29 +1187,21 @@ main(argc, argv)
      */
     DEBUG(DBG_HASH_LO, "main: free memory\n");
     /* free the hash table elements */
-    cur = walk_hash((struct hash *) NULL, table);
-    while (cur) {
-	struct hash *next;		/* pointer to the next element */
-
-	next = walk_hash(cur, table);	/* find the next entry */
-	free_hash_element(cur, 0, table); /* free the current without deletion */
-	cur = next;
+    for (cur=walk_hash((struct hash *)NULL,table);
+	 cur != NULL;
+	 cur=walk_hash(cur,table))
+    {
+	free_hash_element(cur, 0, table); /* free without deletion */
     }
-    free_hash_table(table);	/* free up the header memory */
-    free_block(table_block);
-
+    free_hash_table(table);	/* free up the memory */
     DEBUG(DBG_HASH_LO, "main: free memory ignore case\n");
-    cur = walk_hash((struct hash *) NULL, tableic);
-    while (cur) {
-	struct hash *next;		/* pointer to the next element */
-
-	next = walk_hash(cur, tableic);	/* find the next entry */
-	free_hash_element(cur, 0, tableic); /* free the current without deletion */
-	cur = next;
+    for (cur=walk_hash((struct hash *)NULL,tableic);
+	 cur != NULL;
+	 cur=walk_hash(cur,tableic))
+    {
+	free_hash_element(cur, 0, tableic); /* free without deletion */
     }
-    free_hash_table(tableic);	/* free up the header memory */
-    free_block(tableic_block);
-
+    free_hash_table(tableic);	/* free up the memory */
     DEBUG(DBG_HASH_LO, "main: end\n");
     exit(EX_OK);
 }
@@ -1275,13 +1216,13 @@ test(buf, table)
 {
     register struct hash *cur;	/* the current hash entry */
     char *data;			/* the data we stored */
-    size_t datalen;		/* length of data */
-    size_t buflen;		/* length of the buffer string + NULL */
+    int datalen;		/* length of data */
+    int buflen;			/* length of the buffer string + NULL */
     int i;			/* index */
     register int caseflag;	/* 0 ==> use strcmp, 1 ==> strcmpic */
 
     /* test adding */
-    buflen = strlen(buf) + 1;
+    buflen = strlen(buf)+1;
     i = add_to_hash(buf, buf, buflen, table);
     DEBUG2(DBG_HASH_LO, "test: <%s> add: %d\n", buf, i);
 
@@ -1388,20 +1329,20 @@ dump_hash_table(table)
     /*
      * check the hash chains
      */
-    for (i = 0; i < TABLE_LEN; ++i) {
+    for (i=0; i < TABLE_LEN; ++i) {
 	DEBUG1(DBG_HASH_HI, "dump: slot[%d]:", i);
 	/* check for empty slots */
 	if (empty_slot(table->slot[i])) {
 	    DEBUG(DBG_HASH_HI, "Empty\n");
 	    continue;
 	}
-	for (cur = table->slot[i]; cur; cur = next_hash(cur, table)) {
+	for (cur=table->slot[i]; cur != NULL; cur=next_hash(cur, table)) {
 	    if (cur->keylen == 0) {
 		DEBUG2(DBG_HASH_HI, " 0x%lx: <%s> :EOC ",
-		       (long) cur, cur->keystr);
+		   (long)cur, cur->keystr);
 	    } else {
 		DEBUG3(DBG_HASH_HI, " 0x%lx: <%s> :0x%lx ",
-		       (long) cur, cur->keystr, (long)cur->succ);
+		   (long)cur, cur->keystr, (long)cur->succ);
 	    }
 	}
 	DEBUG(DBG_HASH_HI, "\n");
@@ -1423,7 +1364,7 @@ hash_file_test(table)
     int caseflag;		/* 0 ==> use strcmp, 1 ==> strcmpic */
     char *template;		/* template for mktemp */
     FILE *stream;		/* the file stream */
-    int table_len;		/*  */
+    char *mktemp();		/* form a temp filename */
 
     /*
      * open up a file to perform hash table operations into
@@ -1432,8 +1373,9 @@ hash_file_test(table)
     strcpy(template, tempname);
     filename = mktemp(template);
     DEBUG1(DBG_HASH_LO, "hash_file_test: using <%s>\n", filename);
-    if (! (stream = fopen(filename, "w"))) {
-	panic(EX_CANTCREAT, "hash_file_test: can not fopen(%s, 'w'): %s", filename, strerror(errno));
+    stream = fopen(filename, "w");
+    if (stream == NULL) {
+	panic(EX_CANTCREAT, "hash_file_test: can not creat <%s>", filename);
     }
 
     /*
@@ -1441,27 +1383,23 @@ hash_file_test(table)
      */
     write_hash_table(table, stream);
     if (fclose(stream) != 0) {
-	panic(EX_IOERR, "hash_file_test: can not fclose(): %s", strerror(errno));
+	panic(EX_IOERR, "hash_file_test: can not fclose\n");
     }
 
     /*
      * reread the hash table into memory
      */
     DEBUG1(DBG_HASH_MID, "hash_file_test: rereading <%s>\n", filename);
-    if (! (stream = fopen(filename, "r"))) {
-	panic(EX_NOINPUT, "hash_file_test: can not fopen(%s, 'r'): %s", filename, strerror(errno));
+    stream = fopen(filename, "r");
+    if (stream == NULL) {
+	panic(EX_CANTCREAT, "hash_file_test: can not reopen <%s>", filename);
     }
     if (fstat(fileno(stream), &buf) < 0) {
-	panic(EX_IOERR, "hash_file_test: can not fstat(%s): ", filename, strerror(errno));
+	panic(EX_IOERR, "hash_file_test: can not stat <%s>", filename);
     }
-    if (buf.st_size > SIZE_T_MAX) {
-	panic(EX_IOERR, "hash_file_test: file <%s> too big to read! (%ld)", filename, (long) buf.st_size);
-    }
-    table_len = buf.st_size;
-    DEBUG2(DBG_HASH_LO, "hash_file_test: about to allocate %d bytes to be read from %s.\n", table_len, filename)
-    table2 = (struct hash_table *) xmalloc(table_len * sizeof(char));
-    if (fread((char *) table2, sizeof(char), table_len, stream) != table_len) {
-	panic(EX_IOERR, "hash_file_test: can not fread(%s): %s", filename, strerror(errno));
+    table2 = (struct hash_table *)xmalloc(buf.st_size * sizeof(char));
+    if (fread((char *)table2,sizeof(char),buf.st_size,stream) != buf.st_size) {
+	panic(EX_IOERR, "hash_file_test: can not reread <%s>", filename);
     }
 
     /*
@@ -1469,13 +1407,13 @@ hash_file_test(table)
      */
     caseflag = table->flags & HASH_CASEFOLD;
     DEBUG(DBG_HASH_MID, "hash_file_test: verify\n");
-    for (cur = walk_hash((struct hash *) NULL, table), cur2 = walk_hash((struct hash *) NULL, table2);
-	 cur && cur2;
-	 cur = walk_hash(cur, table), cur2 = walk_hash(cur2, table2)) {
-	    /*
-	     * compare cur and cur2
-	     */
-	    if (stringcmp(cur->keystr, cur2->keystr, caseflag) != 0) {
+    for (cur=walk_hash((struct hash *)NULL,table),
+	 	cur2=walk_hash((struct hash *)NULL,table2);
+	 cur != NULL || cur2 != NULL;
+	 cur=walk_hash(cur,table), cur2=walk_hash(cur2,table2))
+    {
+	    /* compare cur and cur2 */
+	    if (stringcmp(cur->keystr,cur2->keystr,caseflag) != 0) {
 		panic(EX_SOFTWARE,
 		      "hash_file_test: key mismatch: <%s> != <%s>\n",
 		      cur->keystr, cur2->keystr);
@@ -1491,12 +1429,7 @@ hash_file_test(table)
 		      cur->keystr, cur2->keystr);
 	    }
     }
-    if (cur != NULL && cur2 != NULL) {
-	panic(EX_SOFTWARE,
-	      "hash_file_test: %s longer than %s!\n",
-	      cur ? "cur" : "cur2",
-	      cur ? "cur2" : "cur");
-    }
+
     /*
      * cleanup
      */
@@ -1505,9 +1438,3 @@ hash_file_test(table)
 }
 
 #endif	/* STANDALONE */
-
-/* 
- * Local Variables:
- * c-file-style: "smail"
- * End:
- */
