@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_120:pathto.c,v 1.23 2004/08/17 19:49:12 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2:pathto.c,v 1.9 1996/02/28 06:47:14 woods Exp"
  */
 
 /*
@@ -18,56 +18,18 @@
  *	external functions: main, initialize_state, process_args,
  *	deliver_mail
  */
-
-#include "defs.h"
-
 #include <sys/types.h>
 #include <stdio.h>
-
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif
-
-#ifdef HAVE_STRING_H
-# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
-#  include <memory.h>
-# endif
-# include <string.h>
-#endif
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif
-
-#ifdef __STDC__
-# include <stdarg.h>
-#else
-# include <varargs.h>
-#endif
-
-#if defined(HAVE_UNISTD_H)
-# include <unistd.h>
-#endif
-
+#include "defs.h"
 #include "smail.h"
 #include "main.h"
-#include "parse.h"
-#include "addr.h"
-#include "route.h"
-#include "alloc.h"
-#include "list.h"
-#include "smailstring.h"
-#include "dys.h"
 #include "transport.h"
+#include "addr.h"
 #include "exitcodes.h"
-#include "error.h"
-#include "extern.h"
-#include "debug.h"
-#include "smailport.h"
+#ifndef DEPEND
+# include "error.h"
+# include "extern.h"
+#endif
 
 /* variables local to this file */
 static int bang_paths_only;
@@ -76,25 +38,24 @@ static int direct_and_route;
 static int show_transport;
 
 /* functions local to this file */
-static void print_pathto_addr __P((struct addr *));
-static void pathto_direct_and_route __P((char **, struct addr **, struct addr **, struct addr **));
-static char **pathto_setup __P((int, char **, char *, void(*)(void)));
-static struct addr *pathto_route __P((char *, char **));
-static void pathto_usage __P((void));
-static void uupath_usage __P((void));
+static void print_pathto_addr();
+static void pathto_direct_and_route();
+static char **pathto_setup();
+static struct addr *pathto_route();
+static void pathto_usage();
+static void uupath_usage();
 
 /*
  * pathto - return paths to the given addresses.
  */
 void
-pathto(argc, argv)
-    int argc;
+pathto(argv)
     char **argv;			/* generally, argv from main */
 {
     struct addr *addr;
     char *error;
 
-    argv = pathto_setup(argc, argv, "vdnstD:", pathto_usage);
+    argv = pathto_setup(argv, "vdnstD:", pathto_usage);
     if (direct_and_route) {
 	struct addr *okay;		/* list of routed addrs */
 	struct addr *defer;		/* list of temp failed addrs */
@@ -225,9 +186,8 @@ print_pathto_addr(addr)
  */
 /*ARGSUSED*/
 void
-optto(argc, argv)
-    int argc GCC_UNUSED_HACK;
-    char **argv GCC_UNUSED_HACK;
+optto(argv)
+    char **argv;			/* generally, argv from main */
 {
     (void) fprintf(stderr, "%s: optto not currently supported\n", program);
     exit(EX_UNAVAILABLE);
@@ -237,14 +197,13 @@ optto(argc, argv)
  * uupath - return paths to the given host.
  */
 void
-uupath(argc, argv)
-    int argc;
+uupath(argv)
     char **argv;			/* generally, argv from main */
 {
     struct addr *addr;
     char *error;
 
-    for (argv = pathto_setup(argc, argv, "vstD:", uupath_usage); *argv; argv++) {
+    for (argv = pathto_setup(argv, "vstD:", uupath_usage); *argv; argv++) {
 	char *host_path = build_uucp_route(*argv, &error, 0);
 	char *in_addr;
 
@@ -274,7 +233,7 @@ uupath(argc, argv)
 		(void) fprintf(stderr, "%s: %s: %s\n",
 			       program, *argv, error);
 	    } else {
-		last_part = strrchr(path, '!');
+		last_part = rindex(path, '!');
 		if (last_part == NULL) {
 		    (void) printf("%s\n", addr->next_host);
 		} else {
@@ -292,19 +251,53 @@ uupath(argc, argv)
  * pathto_setup - standard setup for the various pathto routines
  */
 static char **
-pathto_setup(argc, argv, oargs, usage)
-    int argc;
+pathto_setup(argv, oargs, usage)
     char **argv;
     char *oargs;
-    void (*usage) __P((void));
+    void (*usage)();
 {
+    char *error;
+    int argc;
     extern char *optarg;
-    extern int optind;
+    extern int optind, opterr;
     int c;
     char **argp;
 
+    /* handle relative configuration file names */
+    config_file = make_lib_fn(config_file);
+
+    /* read in the config and secondary config files */
+    if ((error = read_config_file(config_file)) ||
+	(second_config_file &&
+	   (error = read_config_file(make_lib_fn(second_config_file)))))
+    {
+	panic(EX_OSFILE, "%s", error);
+    }
+
+    /* do the same thing with other variables */
+    director_file = make_lib_fn(director_file);
+    router_file = make_lib_fn(router_file);
+    transport_file = make_lib_fn(transport_file);
+    method_dir = make_lib_fn(method_dir);
+    copying_file = make_lib_fn(copying_file);
+    smail = make_lib_fn(smail);
+
+    /* read the other configuration files */
+    if ((error = read_transport_file()) ||
+	(error = read_router_file()) ||
+	(error = read_director_file()))
+    {
+	panic(EX_OSFILE, "%s", error);
+    }
+
+    /* All errors go to the terminal */
+    error_processing = TERMINAL;
 
     --argv;				/* go back to program name */
+    initialize_state();
+
+    /* count arguments, why does getopt really need this? */
+    for (argc = 0; argv[argc]; argc++) ;
 
     disable_smarthost = TRUE;
     bang_paths_only = TRUE;
@@ -351,8 +344,7 @@ pathto_setup(argc, argv, oargs, usage)
     }
 
     for (argp = &argv[optind]; *argp; argp++) {
-	strip_rfc822_comments(*argp);	/* XXX should we really do this!?!?! */
-	strip_rfc822_whitespace(*argp);
+	strip_rfc822_comments(*argp);
     }
 
     /* setup all of the hostname information */
@@ -370,7 +362,7 @@ pathto_route(in_addr, error)
     struct addr *defer = NULL;
     struct addr *fail = NULL;
     struct addr *done = NULL;
-    int form =  PARSE_ERROR;
+    int form;
 
     cur->in_addr = in_addr;
     if (disable_smarthost) {
@@ -451,9 +443,3 @@ uupath_usage()
 {
     (void) fprintf(stderr, "usage: %s [-vst] addr ...\n", program);
 }
-
-/* 
- * Local Variables:
- * c-file-style: "smail"
- * End:
- */
