@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_121:appendfile.c,v 1.62 2005/08/31 17:53:03 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2:appendfile.c,v 1.30 1996/02/26 18:39:08 woods Exp"
  */
 
 /*
@@ -61,48 +61,18 @@
  *	If a file may be opened over NFS, care must be taken that smail is
  *	running as the desired user during the entire transaction of
  *	opening the mailbox, locking the mailbox and writing out th
- *	message.  At least this is true of NFS in SunOS3.x.  So, the best
- *	answer is:  Don't use NFS.  Do local delivery on the file server.
+ *	message.  At least this is true of NFS in SunOS3.x.  Define
+ *	HAVE_SETEUID to get the necessary behavior.
  */
 
-#include "defs.h"
-
-#include <sys/types.h>
-#include <sys/stat.h>
+#define NEED_SOCKETS
 #include <stdio.h>
 #include <errno.h>
+#include <sys/types.h>
 #include <pwd.h>
 #include <grp.h>
-
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif
-
-#ifdef HAVE_STRING_H
-# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
-#  include <memory.h>
-# endif
-# include <string.h>
-#endif
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif
-
-#ifdef __STDC__
-# include <stdarg.h>
-#else
-# include <varargs.h>
-#endif
-
-#if defined(HAVE_PATHS_H)
-# include <paths.h>
-#endif
-
+#include <sys/stat.h>
+#include "defs.h"
 #if defined(UNIX_SYS5) || defined(POSIX_OS)
 # include <fcntl.h>
 #endif
@@ -112,52 +82,57 @@
 #ifndef HAVE_BSD_NETWORKING
 # undef HAVE_COMSAT
 #endif
-
-#if defined(HAVE_UNISTD_H)
-# include <unistd.h>
-#endif
-
 #include "../smail.h"
-#include "../alloc.h"
-#include "../list.h"
-#include "../smailsock.h"
-#include "../main.h"
+#include "../smailconf.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../log.h"
 #include "../spool.h"
 #include "../child.h"
-#include "../exitcodes.h"
-#include "../smailstring.h"
-#include "../dys.h"
 #include "../transport.h"
-#include "../smailconf.h"
+#include "../exitcodes.h"
+#include "../dys.h"
 #include "appendfile.h"
-#include "../extern.h"
-#include "../debug.h"
-#include "../error.h"
-#include "../smailport.h"
+#ifndef DEPEND
+# include "../extern.h"
+# include "../debug.h"
+# include "../error.h"
+#endif
+
+/* whence values for lseek(2) */
+#ifndef SEEK_SET
+# define SEEK_SET	0	/* set file offset to offset */
+#endif
+#ifndef SEEK_CUR
+# define SEEK_CUR	1	/* set file offset to current plus offset */
+#endif
+#ifndef SEEK_END
+# define SEEK_END	2	/* set file offset to EOF plus offset */
+#endif
+
+/* variables imported from libc */
+extern int errno;
 
 /* functions local to this file */
-static void get_appendfile_ugid __P((struct transport *, struct addr *, uid_t *, gid_t *));
-static time_t get_atime __P((char *));
-static void restore_atime __P((char *, time_t));
+static void get_appendfile_ugid();
+static time_t get_atime();
+static void restore_atime();
 #ifdef HAVE_COMSAT
-static void notify_comsat __P((struct transport *, char *, off_t));
+static void notify_comsat();
 #endif
 
 static struct attr_table appendfile_attributes[] = {
-    { "file", t_string, 0, NULL, NULL, OFFSET(appendfile_private, file) },
-    { "dir", t_string, 0, NULL, NULL, OFFSET(appendfile_private, dir) },
-    { "user", t_string, 0, NULL, NULL, OFFSET(appendfile_private, user) },
-    { "group", t_string, 0, NULL, NULL, OFFSET(appendfile_private, group) },
-    { "prefix", t_string, 0, NULL, NULL, OFFSET(appendfile_private, prefix) },
-    { "suffix", t_string, 0, NULL, NULL, OFFSET(appendfile_private, suffix) },
-    { "mode", t_mode, 0, NULL, NULL, OFFSET(appendfile_private, mode) },
-    { "append_as_user", t_boolean, 0, NULL, NULL, APPEND_AS_USER },
-    { "expand_user", t_boolean, 0, NULL, NULL, APPEND_EXPAND_USER },
-    { "check_user", t_boolean, 0, NULL, NULL, APPEND_CHECK_USER },
-    { "notify_comsat", t_boolean, 0, NULL, NULL, APPEND_COMSAT },
+    { "file", t_string, NULL, NULL, OFFSET(appendfile_private, file) },
+    { "dir", t_string, NULL, NULL, OFFSET(appendfile_private, dir) },
+    { "user", t_string, NULL, NULL, OFFSET(appendfile_private, user) },
+    { "group", t_string, NULL, NULL, OFFSET(appendfile_private, group) },
+    { "prefix", t_string, NULL, NULL, OFFSET(appendfile_private, prefix) },
+    { "suffix", t_string, NULL, NULL, OFFSET(appendfile_private, suffix) },
+    { "mode", t_mode, NULL, NULL, OFFSET(appendfile_private, mode) },
+    { "append_as_user", t_boolean, NULL, NULL, APPEND_AS_USER },
+    { "expand_user", t_boolean, NULL, NULL, APPEND_EXPAND_USER },
+    { "check_user", t_boolean, NULL, NULL, APPEND_CHECK_USER },
+    { "notify_comsat", t_boolean, NULL, NULL, APPEND_COMSAT },
 };
 
 static struct attr_table *end_appendfile_attributes = ENDTABLE(appendfile_attributes);
@@ -181,14 +156,17 @@ tpd_appendfile(addr, succeed, defer, fail)
     FILE *f;				/* open file */
     char *user;				/* expanded username */
     char *save_user = NULL;		/* save old username */
-    uid_t uid;				/* uid for creating file */
-    gid_t gid;				/* gid for creating file */
-    time_t atime_saved = 0;		/* preserved access time */
+    int uid;				/* uid for creating file */
+    int gid;				/* gid for creating file */
+    time_t atime_saved;			/* preserved access time */
 #ifdef HAVE_COMSAT
-    off_t msg_offset;			/* offset of mbox end */
+    long msg_offset;			/* offset of mbox end */
+#endif
+#ifdef HAVE_SETEUID
+    int save_gid;			/* saved gid from before setegid() */
 #endif
 
-    DEBUG1(DBG_DRIVER_HI, "tpd_appendfile called: addr = %v\n", addr->in_addr);
+    DEBUG1(DBG_DRIVER_HI, "tpd_appendfile called: addr = %s\n", addr->in_addr);
     priv = (struct appendfile_private *)tp->private;
 
     /* subject addresses to retry interval and duration limits */
@@ -228,11 +206,11 @@ tpd_appendfile(addr, succeed, defer, fail)
 	    register char *s;
 
 	    if (addr->parent) {
-		s = xprintf("transport %v: could not expand user (parent %v)",
+		s = xprintf("transport %s: could not expand user (parent %s)",
 			    tp->name,
 			    addr->parent->in_addr);
 	    } else {
-		s = xprintf("transport %v: could not expand user",
+		s = xprintf("transport %s: could not expand user",
 			    tp->name);
 	    }
 	    er = note_error(ERR_NPOWNER|ERR_128, s);
@@ -247,7 +225,7 @@ tpd_appendfile(addr, succeed, defer, fail)
 	 * make sure the username can't cause an expansion to move to
 	 * another directory
 	 */
-	if (strchr(addr->next_addr, '/')) {
+	if (index(addr->next_addr, '/')) {
 	    /*
 	     * ERR_129 - username contains /
 	     *
@@ -267,7 +245,7 @@ tpd_appendfile(addr, succeed, defer, fail)
 	    register struct error *er;
 
 	    er = note_error(ERR_NSOWNER|ERR_129,
-			    xprintf("transport %v: username contains /",
+			    xprintf("transport %s: username contains /",
 				    tp->name));
 	    insert_addr_list(addr, fail, er);
 	    return;
@@ -297,7 +275,7 @@ tpd_appendfile(addr, succeed, defer, fail)
 	register struct error *er;
 
 	er = note_error(ERR_CONFERR|ERR_130,
-			xprintf("transport %v: file or dir attribute required",
+			xprintf("transport %s: file or dir attribute required",
 				tp->name));
 	insert_addr_list(addr, defer, er);
 	return;
@@ -325,7 +303,7 @@ tpd_appendfile(addr, succeed, defer, fail)
 	register struct error *er;
 
 	er = note_error(ERR_CONFERR|ERR_131,
-			xprintf("transport %v: failed to expand %v",
+			xprintf("transport %s: failed to expand %s",
 				tp->name, priv->file));
 	insert_addr_list(addr, defer, er);
 	return;
@@ -352,7 +330,7 @@ tpd_appendfile(addr, succeed, defer, fail)
 	register struct error *er;
 
 	er = note_error(ERR_CONFERR|ERR_132,
-			xprintf("transport %v: pathname not absolute",
+			xprintf("transport %s: pathname not absolute",
 				tp->name));
 	insert_addr_list(addr, defer, er);
 	return;
@@ -365,7 +343,7 @@ tpd_appendfile(addr, succeed, defer, fail)
      * special case /dev/null, and don't do any more if dont_deliver
      * is set
      */
-    if (dont_deliver || EQ(fn, _PATH_DEVNULL)) {
+    if (dont_deliver || EQ(fn, "/dev/null")) {
 	insert_addr_list(addr, succeed, (struct error *)NULL);
 	return;
     }
@@ -379,63 +357,49 @@ tpd_appendfile(addr, succeed, defer, fail)
     if (priv->file) {
 	atime_saved = get_atime(fn);
     }
-/*
- * Date: Wed, 12 Feb 1997 10:11:38 +0000 (GMT)
- * From: Philip Hazel <ph10@cus.cam.ac.uk>
- * To: exim-users@lists.cam.ac.uk
- * Subject: Re: Solaris 2.5 NFS apparent bug - ideas for workaround?
- * In-Reply-To: <Pine.SOL.3.95.970206092004.1374D-100000@taurus.cus.cam.ac.uk>
- * Message-ID: <Pine.SOL.3.95.970212100950.7535N-100000@taurus.cus.cam.ac.uk>
- * 
- * On Thu, 6 Feb 1997 I wrote:
- * 
- * > We have discovered what appears to be a bug in the client implementation 
- * > of NFS in Solaris 2.5. If it isn't a bug, it's a worrying incompatibility.
- * > It can affect versions of Exim that are being used to do local 
- * > deliveries into NFS-mounted files.
- * 
- * <snip>
- * 
- * I have been silly. Exim used to proceed as follows:
- * 
- *   open mailbox    
- *   create lock file 
- *   lock open mailbox with fcntl  
- *           
- * I don't know why I did it this way; I think I had "open the file before 
- * locking" stuck in my brain. Anyway, I have now changed it so that it 
- * does                                                                     
- * 
- *   create lock file
- *   open mailbox 
- *   lock open mailbox with fcntl  
- *      
- * Thus it should no longer be possible for two different Exims to have  
- * the mailbox open simultaneously, and the attribute cacheing problem 
- * should be avoided.                                                         
- * 
- * I have looked at the code of Pine, and it seems to do things in this 
- * order as well, so that should interlock safely too. I don't know about 
- * the other MUAs.                                                          
- * 
- * I will put something in the Exim manual pointing out that lock files 
- * should always be used when delivering over NFS if more than one host is 
- * involved.                                                               
- * 
- * --
- * Philip Hazel                   University Computing Service,
- * ph10@cus.cam.ac.uk             New Museums Site, Cambridge CB2 3QG,
- * P.Hazel@ucs.cam.ac.uk          England.  Phone: +44 1223 334714
- */
+
     fn = COPY_STRING(fn);
+#ifdef HAVE_SETEUID
+    /*
+     * If we are using NFS, then assume that the seteuid call is available
+     * and become the desired user here.  It is not enough to merely be
+     * the desired user when opening the file (what a pain!).
+     */
+    /* order is important here */
+    save_gid = getegid();
+    (void) setegid(gid);
+    (void) seteuid(uid);
+
+    f = NULL;
+    if (priv->file) {
+	int fd;
+
+	/* open and lock the mailbox file */
+	temp_fn = COPY_STRING(fn);	/* copy expanded string */
+	fd = open(temp_fn, O_WRONLY|O_APPEND|O_CREAT, priv->mode);
+	if (fd >= 0) {
+	    f = fdopen(fd, "a");
+	}
+    } else {
+	int fd;
+
+	fn = COPY_STRING(fn);
+	temp_fn = xprintf("%s/temp.%d", fn, getpid());
+	fd = open(temp_fn, O_WRONLY|O_CREAT, priv->mode);
+	if (fd >= 0) {
+	    f = fdopen(fd, "a");
+	}
+    }
+#else	/* not HAVE_SETEUID */
     if (priv->file) {
 	/* open and lock the mailbox file */
 	temp_fn = COPY_STRING(fn);	/* copy expanded string */
 	f = fopen_as_user(fn, "a", uid, gid, priv->mode);
     } else {
-	temp_fn = xprintf("%s/temp.%ld", fn, (long int) getpid());
+	temp_fn = xprintf("%s/temp.%d", fn, getpid());
 	f = fopen_as_user(temp_fn, "w", uid, gid, priv->mode);
     }
+#endif	/* not HAVE_SETEUID */
     DEBUG1(DBG_DRIVER_LO, "  appendfile: write to file %s\n", temp_fn);
 
     if (f == NULL) {
@@ -464,12 +428,18 @@ tpd_appendfile(addr, succeed, defer, fail)
 	register struct error *er;
 
 	er = note_error(ERR_NPOWNER|ERR_133,
-			xprintf("transport %v: failed to open output file: %s",
+			xprintf("transport %s: failed to open output file: %s",
 				tp->name, strerror(errno)));
 	insert_addr_list(addr, fail, er);
 	xfree(temp_fn);			/* no longer need filename */
 	xfree(fn);
 
+#ifdef HAVE_SETEUID
+	/* order is important here */
+	(void) seteuid(0);
+	(void) setegid(save_gid);
+#endif
+
 	return;
     }
 
@@ -496,7 +466,7 @@ tpd_appendfile(addr, succeed, defer, fail)
 	register struct error *er;
 
 	er = note_error(ERR_134,
-			xprintf("transport %v: failed to lock mailbox",
+			xprintf("transport %s: failed to lock mailbox",
 				tp->name));
 	insert_addr_list(addr, defer, er);
 	(void) fclose(f);
@@ -506,6 +476,12 @@ tpd_appendfile(addr, succeed, defer, fail)
 	xfree(temp_fn);
 	xfree(fn);
 
+#ifdef HAVE_SETEUID
+	/* order is important here */
+	(void) seteuid(0);
+	(void) setegid(save_gid);
+#endif
+
 	return;
     }
 
@@ -518,7 +494,7 @@ tpd_appendfile(addr, succeed, defer, fail)
     /* write out the message */
     if ((priv->prefix && 
 	 fputs(expand_string(priv->prefix, addr, (char *) NULL, (char *) NULL), f) == EOF) ||
-	write_message(f, addr) != SUCCEED ||
+	write_message(f, tp, addr) != SUCCEED ||
 	(priv->suffix && fputs(priv->suffix, f) == EOF) ||
 	fflush(f) == EOF)
     {
@@ -542,7 +518,7 @@ tpd_appendfile(addr, succeed, defer, fail)
 	register struct error *er;
 
 	er = note_error(ERR_135,
-			xprintf("transport %v: write to mailbox failed: %s",
+			xprintf("transport %s: write to mailbox failed: %s",
 				tp->name, strerror(errno)));
 	insert_addr_list(addr, defer, er);
 	(void) fclose(f);
@@ -552,6 +528,12 @@ tpd_appendfile(addr, succeed, defer, fail)
 	xfree(temp_fn);
 	xfree(fn);
 
+#ifdef HAVE_SETEUID
+	/* order is important here */
+	(void) seteuid(0);
+	(void) setegid(save_gid);
+#endif
+
 	return;
     }
 
@@ -612,13 +594,19 @@ tpd_appendfile(addr, succeed, defer, fail)
 	    register struct error *er;
 
 	    er = note_error(ERR_NPOSTMAST|ERR_136,
-		      xprintf("transport %v: rename failed for queue file: %s",
+		      xprintf("transport %s: rename failed for queue file: %s",
 			      tp->name, strerror(errno)));
 	    insert_addr_list(addr, fail, er);
 	    xfree(temp_fn);
 	    xfree(perm_fn);
 	    xfree(fn);
 
+#ifdef HAVE_SETEUID
+	/* order is important here */
+	(void) seteuid(0);
+	(void) setegid(save_gid);
+#endif
+
 	    return;
 	}
 	xfree(perm_fn);
@@ -629,6 +617,12 @@ tpd_appendfile(addr, succeed, defer, fail)
     xfree(temp_fn);			/* free temporary storage */
     xfree(fn);
 
+#ifdef HAVE_SETEUID
+	/* order is important here */
+	(void) seteuid(0);
+	(void) setegid(save_gid);
+#endif
+
     return;
 }
 
@@ -651,7 +645,6 @@ tpb_appendfile(tp, attrs)
 	0666,				/* mode */
     };
     struct appendfile_private *priv;	/* new appendfile_private structure */
-    char *fn = xprintf("appendfile transport: %v", tp->name);
 
     /* copy the template private data */
     priv = (struct appendfile_private *)xmalloc(sizeof(*priv));
@@ -667,13 +660,11 @@ tpb_appendfile(tp, attrs)
 			    attrs,
 			    &tp->flags,
 			    appendfile_attributes,
-			    end_appendfile_attributes,
-			    fn);
-    xfree(fn);
+			    end_appendfile_attributes);
+
     if (error) {
 	return error;
     }
-
     return NULL;
 }
 
@@ -704,8 +695,8 @@ static void
 get_appendfile_ugid(tp, addr, uid, gid)
     struct transport *tp;		/* associated transport structure */
     struct addr *addr;			/* associated addr structures */
-    uid_t *uid;				/* store uid here */
-    gid_t *gid;				/* store gid here */
+    int *uid;				/* store uid here */
+    int *gid;				/* store gid here */
 {
     struct appendfile_private *priv = (struct appendfile_private *)tp->private;
     /*
@@ -718,11 +709,11 @@ get_appendfile_ugid(tp, addr, uid, gid)
 	    *uid = nobody_uid;
 	}
     } else {
-	struct passwd *pw = getpwbyname(FALSE, priv->user);
+	struct passwd *pw = getpwbyname(priv->user);
 
 	if (pw == NULL) {
-	    write_log(WRITE_LOG_PANIC,
-		      "transport %v: warning: user %v unknown, using nobody",
+	    write_log(LOG_PANIC,
+		      "transport %s: warning: user %s unknown, using nobody",
 		      tp->name, priv->user);
 	    priv->user = NULL;
 	    *uid = nobody_uid;
@@ -737,8 +728,8 @@ get_appendfile_ugid(tp, addr, uid, gid)
 	struct group *gr = getgrbyname(priv->group);
 
 	if (gr == NULL) {
-	    write_log(WRITE_LOG_PANIC,
-		      "transport %v: warning: group %v unknown, ignored",
+	    write_log(LOG_PANIC,
+		      "transport %s: warning: group %s unknown, ignored",
 		      tp->name, priv->group);
 	    priv->group = NULL;
 	} else {
@@ -812,9 +803,9 @@ restore_atime(fname, atime)
 /* ARGSUSED */
 static void
 notify_comsat(tp, mbox, offset)
-    struct transport *tp __attribute__((unused));
-    char *mbox;
-    off_t offset;
+struct transport *tp;				/* UNUSED! */
+char *mbox;
+off_t offset;
 {
     static char me[] = "notify_comsat";
     struct sockaddr_in sa;
@@ -824,15 +815,15 @@ notify_comsat(tp, mbox, offset)
     int fd, ret;
 
     if ((hp = gethostbyname("localhost")) == NULL) {
-	DEBUG1(DBG_DRIVER_LO, "%s: localhost: unknown host\n", me);
+	DEBUG1(DBG_DRIVER_LO, "%s: localhost: unknown host", me);
         return;
     }
     if (hp->h_addrtype != AF_INET) {
-	DEBUG1(DBG_DRIVER_LO, "%s: localhost: not TCP/IP\n", me);
+	DEBUG1(DBG_DRIVER_LO, "%s: localhost: not TCP/IP", me);
 	return;
     }
     if ((sp = getservbyname("biff", "udp")) == NULL) {
-	DEBUG1(DBG_DRIVER_LO, "%s: biff/udp: unknown service\n", me);
+	DEBUG1(DBG_DRIVER_LO, "%s: biff/udp: unknown service", me);
 	return;
     }
 
@@ -851,13 +842,8 @@ notify_comsat(tp, mbox, offset)
 	++base;
     }
     msg = xprintf("%s@%d\n", base, offset);
-    DEBUG2(DBG_DRIVER_HI, "%s: sending msg: '%v'\n", me, msg);
-    /* XXX some systems have arg-3 as size_t, others as int !!!! */
-#if defined(HAVE_SENDTO_WITH_3RD_PARAM_INT)
-    ret = sendto(fd, msg, (int) (strlen(msg) + 1), 0, (struct sockaddr *) &sa, sizeof(sa));
-#else
-    ret = sendto(fd, msg, (size_t) (strlen(msg) + 1), 0, (struct sockaddr *) &sa, (socklen_t) sizeof(sa));
-#endif
+    DEBUG2(DBG_DRIVER_HI, "%s: sending msg: %s", me, msg);
+    ret = sendto(fd, msg, strlen(msg) + 1, 0, (struct sockaddr *)&sa, sizeof(sa));
     xfree(msg);
     if (ret == -1) {
 	DEBUG2(DBG_DRIVER_LO, "%s: send: %s\n", me, strerror(errno));
@@ -867,9 +853,3 @@ notify_comsat(tp, mbox, offset)
 }
 
 #endif /* HAVE_COMSAT */
-
-/* 
- * Local Variables:
- * c-file-style: "smail"
- * End:
- */
