@@ -1,9 +1,5 @@
 #! /bin/sh
-:
-#ident	"@(#)smail/util:RELEASE-3_2_0_120:smailbug.sh,v 1.13 2004/05/15 23:46:36 woods Exp"
-#
 # Submit a problem report to a GNATS site.
-#
 # Copyright (C) 1993 Free Software Foundation, Inc.
 # Contributed by Brendan Kehoe (brendan@cygnus.com), based on a
 # version written by Heinz G. Seidl (hgs@cygnus.com).
@@ -19,11 +15,10 @@
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
-# along with GNU GNATS; see the file COPYING.  If not, write to the
-# Free Software Foundation; 59 Temple Place; Suite 330; Boston, MA;
-# 02111 USA
+# along with GNU GNATS; see the file COPYING.  If not, write to
+# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 # The version of this send-pr.
 VERSION=3.95
@@ -60,84 +55,43 @@ DEFAULT_ORGANIZATION="net"
 # What mailer to use.  This must come after the config file, since it is
 # host-dependent.
 if [ -f X_SMAIL_NAME_X ]; then
-	MAIL_AGENT="X_SMAIL_NAME_X -oi -t"
+  MAIL_AGENT="X_SMAIL_NAME_X -oi -t"
 elif [ -f /usr/sbin/sendmail ]; then
-	MAIL_AGENT="/usr/sbin/sendmail -oi -t"
+  MAIL_AGENT="/usr/sbin/sendmail -oi -t"
 else
-	MAIL_AGENT="/usr/lib/sendmail -oi -t"
+  MAIL_AGENT="/usr/lib/sendmail -oi -t"
 fi
 
 # How to read the passwd database.
 PASSWD="cat /etc/passwd"
 
-if [ -z "$TMPDIR" ]; then
-	TMPDIR="/tmp"
-fi
-export TMPDIR
+ECHON=sysv
 
-# XXX should use mktemp, but we don't have a portable one....
-# we could use a private subdir, but the risks are so low....
-#
-TEMP=$TMPDIR/p$$
-BAD=$TMPDIR/pbad$$
-REF=$TMPDIR/pf$$
-
-# Catch some signals. ($xs kludge needed by Sun /bin/sh)
-xs=0
-trap 'rm -f $BAD; exit $xs' 0
-trap 'echo "$COMMAND: Aborting ..."; rm -f $REF $TEMP ; exit' 1 2 3 13 15
-
-# XXX this "echo" determination is cut&paste from my .profile <woods@planix.com>
-HAVEPRINT=false ; export HAVEPRINT
-if expr "`type print`" : '^print is a shell builtin$' >/dev/null 2>&1 ; then
-	HAVEPRINT=true
-fi
-HAVEPRINTF=false ; export HAVEPRINTF
-if expr "`type printf`" : '^printf is a shell builtin$' >/dev/null 2>&1 ; then
-	HAVEPRINTF=true
-elif expr "`type printf`" : '.* is .*/printf$' >/dev/null 2>&1 ; then
-	HAVEPRINTF=true
-fi
-# always use ``$echo'' if any of the other variables are used...
-#	$nl - print a newline (always required at end of line if desired)
-#	$n - option to turn off final newline
-#	$c - escape sequence to turn off final newline
-# usage for a prompt is:
-#	$echo $n "prompt: $c"
-# and for a normal line
-#	$echo "message$nl"
-#
-if $HAVEPRINT ; then
-	# use ``$echo'' if any of the other variables...
-	echo=print
-	nl='\n'
-	n='-n'
-	# XXX in theory '\c' is equivalent of '-n' in most shells
-	c=''
-elif $HAVEPRINTF ; then
-	# use ``$echo'' if any of the other variables...
-	echo=printf
-	nl='\n'
-	n=''
-	c=''
+if [ $ECHON = bsd ] ; then
+  ECHON1="echo -n"
+  ECHON2=
+elif [ $ECHON = sysv ] ; then
+  ECHON1=echo
+  ECHON2='\c'
 else
-	echo=echo
-	(echo "hi there\c" ; echo " ") >$TMPDIR/echotmp
-	# Configure checks to make sure grep returns a status...
-	if grep c $TMPDIR/echotmp >/dev/null 2>&1 ; then
-		nl=''
-		n='-n'
-		c=''
-	else
-		nl='\n'
-		n=''
-		c='\c'
-	fi
-	rm -f $TMPDIR/echotmp
+  ECHON1=echo
+  ECHON2=
 fi
 
 #
 
+if [ -z "$TMPDIR" ]; then
+  TMPDIR=/tmp
+else
+  if [ "`echo $TMPDIR | grep '/$'`" != "" ]; then
+    TMPDIR="`echo $TMPDIR | sed -e 's,/$,,'`"
+  fi
+fi
+
+TEMP=$TMPDIR/p$$
+BAD=$TMPDIR/pbad$$
+REF=$TMPDIR/pf$$
+
 # find a user name
 if [ "$LOGNAME" = "" ]; then
 	if [ "$USER" != "" ]; then
@@ -160,8 +114,7 @@ elif [ -f $HOME/.fullname ]; then
 else
   # Must use temp file due to incompatibilities in quoting behavior
   # and to protect shell metacharacters in the expansion of $LOGNAME
-  # XXX I don't really believe that is true....
-  $PASSWD | grep "^${LOGNAME}:" | awk -F: '{print $5}' | sed -e 's/,.*//' > $TEMP
+  $PASSWD | grep "^$LOGNAME:" | awk -F: '{print $5}' | sed -e 's/,.*//' > $TEMP
   ORIGINATOR="`cat $TEMP`"
   rm -f $TEMP
 fi
@@ -196,10 +149,10 @@ SYSTEM=`( [ -f /bin/uname ] && /bin/uname -a ) || \
         ( [ -f /usr/bin/uname ] && /usr/bin/uname -a ) || echo ""`
 ARCH=`[ -f /bin/arch ] && /bin/arch`
 MACHINE=`[ -f /bin/machine ] && /bin/machine`
-SMAIL_VERSION=`X_SMAIL_NAME_X -v -V`
 
 COMMAND=`echo $0 | sed -e 's,.*/,,'`
-USAGE="Usage: $COMMAND [-PVL] [-f filename] [--version]"
+USAGE="Usage: $COMMAND [-PVL] [-t address] [-f filename] [--request-id] 
+[--version]"
 REMOVE=
 BATCH=
 
@@ -210,13 +163,13 @@ while [ $# -gt 0 ]; do
 ## 	shift ; GNATS_ADDR="$1"
 ## 	EXPLICIT_GNATS_ADDR=true
 ##         ;;
-    -f | --file) if [ $# -eq 1 ]; then echo "$USAGE"; exit 1; fi
-	shift ; IN_FILE="$1"
-	if [ "$IN_FILE" != "-" -a ! -r "$IN_FILE" ]; then
-	  echo "$COMMAND: cannot read $IN_FILE"
-	  exit 1
-	fi
-	;;
+##     -f | --file) if [ $# -eq 1 ]; then echo "$USAGE"; exit 1; fi
+## 	shift ; IN_FILE="$1"
+## 	if [ "$IN_FILE" != "-" -a ! -r "$IN_FILE" ]; then
+## 	  echo "$COMMAND: cannot read $IN_FILE"
+## 	  exit 1
+## 	fi
+## 	;;
     -b | --batch) BATCH=true ;;
     -p | -P | --print) PRINT=true ;;
     -L | --list) FORMAT=norm ;;
@@ -270,7 +223,7 @@ done
 ##   exit 1
 ## fi
 
-CATEGORIES="smail smail-util smail-pd smail-contrib smail-samples"
+CATEGORIES="query-smail smail smail-bugs smail-contrib smail-docs smail-enhance smail-ports smail-security smail-unknown"
 
 ## if [ -z "$CATEGORIES" ]; then
 ##   echo "$COMMAND: the categories list for $GNATS_SITE was empty!"
@@ -285,11 +238,11 @@ case "$FORMAT" in
   norm) l=`echo "$CATEGORIES" | \
 	awk 'BEGIN {max = 0; } { if (length($0) > max) { max = length($0); } }
 	     END {print max + 1;}'`
-	lcnt=`expr 70 / $l`
-	if [ $lcnt -eq 0 ]; then c=1; fi
+	c=`expr 70 / $l`
+	if [ $c -eq 0 ]; then c=1; fi
 	echo "$CATEGORIES" | \
         awk 'BEGIN {print "Known categories:"; i = 0 }
-          { printf ("%-'$l'.'$l's", $0); if ((++i % '$lcnt') == 0) { print "" } }
+          { printf ("%-'$l'.'$l's", $0); if ((++i % '$c') == 0) { print "" } }
             END { print ""; }'
         exit 0
         ;;
@@ -309,6 +262,11 @@ DESCRIPTION_C='<precise description of the problem (multiple lines)>'
 HOW_TO_REPEAT_C='<code/input/activities to reproduce the problem (multiple lines)>'
 FIX_C='<how to correct or work around the problem, if known (multiple lines)>'
 
+# Catch some signals. ($xs kludge needed by Sun /bin/sh)
+xs=0
+trap 'rm -f $REF $TEMP; exit $xs' 0
+trap 'echo "$COMMAND: Aborting ..."; rm -f $REF $TEMP; xs=1; exit' 1 2 3 13 15
+
 ## # If they told us to use a specific file, then do so.
 ## if [ -n "$IN_FILE" ]; then
 ##   if [ "$IN_FILE" = "-" ]; then
@@ -344,11 +302,11 @@ FIX_C='<how to correct or work around the problem, if known (multiple lines)>'
     for file in $TEMP $REF ; do
       cat  > $file << '__EOF__'
 SEND-PR: -*- send-pr -*-
-SEND-PR: Lines starting with 'SEND-PR' will be removed automatically, as
-SEND-PR: will all comments (text enclosed in '<' and '>').
+SEND-PR: Lines starting with `SEND-PR' will be removed automatically, as
+SEND-PR: will all comments (text enclosed in `<' and `>').
 SEND-PR: 
-SEND-PR: Please consult the manual page 'smailbug(8)', or the Texinfo
-SEND-PR: manual, if you are not sure how to fill out a problem report.
+SEND-PR: Please consult the send-pr man page `send-pr(1)' or the Texinfo
+SEND-PR: manual if you are not sure how to fill out a problem report.
 SEND-PR:
 SEND-PR: Choose from the following categories:
 SEND-PR:
@@ -358,12 +316,12 @@ __EOF__
 	l=`echo "$CATEGORIES" | \
 	awk 'BEGIN {max = 0; } { if (length($0) > max) { max = length($0); } }
 	     END {print max + 1;}'`
-	lcnt=`expr 61 / $l`
-	if [ $lcnt -eq 0 ]; then c=1; fi
+	c=`expr 61 / $l`
+	if [ $c -eq 0 ]; then c=1; fi
 	echo "$CATEGORIES" | \
         awk 'BEGIN {printf "SEND-PR: "; i = 0 }
           { printf ("%-'$l'.'$l's", $0);
-	    if ((++i % '$lcnt') == 0) { printf "\nSEND-PR: " } }
+	    if ((++i % '$c') == 0) { printf "\nSEND-PR: " } }
             END { printf "\nSEND-PR:\n"; }' >> $file
 
       cat >> $file << __EOF__
@@ -390,19 +348,12 @@ ${ORGANIZATION-	$ORGANIZATION_C}
 `[ -n "$SYSTEM" ] && echo System: $SYSTEM`
 `[ -n "$ARCH" ] && echo Architecture: $ARCH`
 `[ -n "$MACHINE" ] && echo Machine: $MACHINE`
-
-Smail:	X_SMAIL_NAME_X
-$SMAIL_VERSION
-
 >Description:
 	$DESCRIPTION_C
-
 >How-To-Repeat:
 	$HOW_TO_REPEAT_C
-
 >Fix:
 	$FIX_C
-
 __EOF__
     done
   fi
@@ -413,16 +364,16 @@ __EOF__
   fi
 
   chmod u+w $TEMP
-##   if [ -z "$REQUEST_ID" ]; then
+  if [ -z "$REQUEST_ID" ]; then
     eval $EDIT $TEMP
-##   else
-##     ed -s $TEMP << '__EOF__'
-## /^Subject/s/^Subject:.*/Subject: request for a customer id/
-## /^>Category/s/^>Category:.*/>Category: send-pr/
-## w
-## q
-## __EOF__
-##  fi
+  else
+    ed -s $TEMP << '__EOF__'
+/^Subject/s/^Subject:.*/Subject: request for a customer id/
+/^>Category/s/^>Category:.*/>Category: send-pr/
+w
+q
+__EOF__
+  fi
 
   if cmp -s $REF $TEMP ; then
     echo "$COMMAND: problem report not filled out, therefore not sent"
@@ -510,7 +461,7 @@ while [ -z "$REQUEST_ID" ]; do
 
   while true; do
     if [ -z "$BATCH" ]; then
-      $echo $n "a)bort, e)dit or s)end? $c"
+      $ECHON1 "a)bort, e)dit or s)end? $ECHON2"
       read input
     else
       if [ $CNT -eq 5 ]; then
